@@ -1,11 +1,12 @@
 #pragma once
 
-#include <QWidget>
+#include <QDialog>
 #include <QString>
+#include <memory>
 
 class Ui_ScriptsTool;
 
-class ScriptLogWindow : public QWidget {
+class ScriptLogWindow : public QDialog {
 	Q_OBJECT
 
 	QString lines;
@@ -24,10 +25,10 @@ public slots:
 	void ScrollChanged(int val);
 };
 
-class ScriptsTool : public QWidget {
+class ScriptsTool : public QDialog {
 	Q_OBJECT
 
-	Ui_ScriptsTool *ui;
+	std::unique_ptr<Ui_ScriptsTool> ui;
 	QWidget *propertiesView = nullptr;
 
 public:
