@@ -1,21 +1,22 @@
 #pragma once
 
 #include <QDialog>
-#include <QTextEdit>
+#include <QPlainTextEdit>
 #include "obs-app.hpp"
 
+#include "ui_OBSLogViewer.h"
+
 class OBSLogViewer : public QDialog {
 	Q_OBJECT
 
-	QPointer<QTextEdit> textArea;
+	std::unique_ptr<Ui::OBSLogViewer> ui;
 
 	void InitLog();
 
 private slots:
 	void AddLine(int type, const QString &text);
-	void ClearText();
-	void ToggleShowStartup(bool checked);
-	void OpenFile();
+	void on_openButton_clicked();
+	void on_showStartup_clicked(bool checked);
 
 public:
 	OBSLogViewer(QWidget *parent = 0);
