@@ -276,9 +276,6 @@ void OBSBasic::on_actionRenameSceneCollection_triggered()
 		return;
 	}
 
-	if (api)
-		api->on_event(OBS_FRONTEND_EVENT_SCENE_COLLECTION_CHANGING);
-
 	oldFile.insert(0, path);
 	oldFile += ".json";
 	os_unlink(oldFile.c_str());
@@ -293,10 +290,8 @@ void OBSBasic::on_actionRenameSceneCollection_triggered()
 	UpdateTitleBar();
 	RefreshSceneCollections();
 
-	if (api) {
-		api->on_event(OBS_FRONTEND_EVENT_SCENE_COLLECTION_LIST_CHANGED);
-		api->on_event(OBS_FRONTEND_EVENT_SCENE_COLLECTION_CHANGED);
-	}
+	if (api)
+		api->on_event(OBS_FRONTEND_EVENT_SCENE_COLLECTION_RENAMED);
 }
 
 void OBSBasic::on_actionRemoveSceneCollection_triggered()
@@ -325,8 +320,8 @@ void OBSBasic::on_actionRemoveSceneCollection_triggered()
 	if (newPath.empty())
 		return;
 
-	QString text = QTStr("ConfirmRemove.Text");
-	text.replace("$1", QT_UTF8(oldName.c_str()));
+	QString text =
+		QTStr("ConfirmRemove.Text").arg(QT_UTF8(oldName.c_str()));
 
 	QMessageBox::StandardButton button = OBSMessageBox::question(
 		this, QTStr("ConfirmRemove.Title"), text);
@@ -372,10 +367,8 @@ void OBSBasic::on_actionRemoveSceneCollection_triggered()
 
 void OBSBasic::on_actionImportSceneCollection_triggered()
 {
-	OBSImporter *imp;
-	imp = new OBSImporter(this);
-	imp->exec();
-	delete imp;
+	OBSImporter imp(this);
+	imp.exec();
 
 	RefreshSceneCollections();
 }
