@@ -97,6 +97,8 @@ class OBSBasicSettings : public QDialog {
 			   DESIGNABLE true)
 	Q_PROPERTY(QIcon hotkeysIcon READ GetHotkeysIcon WRITE SetHotkeysIcon
 			   DESIGNABLE true)
+	Q_PROPERTY(QIcon accessibilityIcon READ GetAccessibilityIcon WRITE
+			   SetAccessibilityIcon DESIGNABLE true)
 	Q_PROPERTY(QIcon advancedIcon READ GetAdvancedIcon WRITE SetAdvancedIcon
 			   DESIGNABLE true)
 
@@ -113,6 +115,7 @@ private:
 	bool audioChanged = false;
 	bool videoChanged = false;
 	bool hotkeysChanged = false;
+	bool a11yChanged = false;
 	bool advancedChanged = false;
 	int pageIndex = 0;
 	bool loading = true;
@@ -120,12 +123,16 @@ private:
 	std::string savedTheme;
 	int sampleRateIndex = 0;
 	int channelIndex = 0;
+	bool llBufferingEnabled = false;
 
 	int lastSimpleRecQualityIdx = 0;
 	int lastServiceIdx = -1;
 	int lastIgnoreRecommended = -1;
 	int lastChannelSetupIdx = 0;
 
+	static constexpr uint32_t ENCODER_HIDE_FLAGS =
+		(OBS_ENCODER_CAP_DEPRECATED | OBS_ENCODER_CAP_INTERNAL);
+
 	OBSFFFormatDesc formats;
 
 	OBSPropertiesView *streamProperties = nullptr;
@@ -190,7 +197,7 @@ private:
 	{
 		return generalChanged || outputsChanged || stream1Changed ||
 		       audioChanged || videoChanged || advancedChanged ||
-		       hotkeysChanged;
+		       hotkeysChanged || a11yChanged;
 	}
 
 	inline void EnableApplyButton(bool en)
@@ -206,6 +213,7 @@ private:
 		audioChanged = false;
 		videoChanged = false;
 		hotkeysChanged = false;
+		a11yChanged = false;
 		advancedChanged = false;
 		EnableApplyButton(false);
 	}
@@ -220,11 +228,15 @@ private:
 
 	bool QueryChanges();
 
-	void LoadEncoderTypes();
+	void ResetEncoders(bool streamOnly = false);
 	void LoadColorRanges();
+	void LoadColorSpaces();
+	void LoadColorFormats();
 	void LoadFormats();
 	void ReloadCodecs(const ff_format_desc *formatDesc);
 
+	void UpdateColorFormatSpaceWarning();
+
 	void LoadGeneralSettings();
 	void LoadStream1Settings();
 	void LoadOutputSettings();
@@ -232,6 +244,7 @@ private:
 	void LoadVideoSettings();
 	void
 	LoadHotkeySettings(obs_hotkey_id ignoreKey = OBS_INVALID_HOTKEY_ID);
+	void LoadA11ySettings(bool presetChange = false);
 	void LoadAdvancedSettings();
 	void LoadSettings(bool changedOnly);
 
@@ -298,12 +311,29 @@ private:
 	void LoadResolutionLists();
 	void LoadFPSData();
 
+	/* a11y */
+	void UpdateA11yColors();
+	void SetDefaultColors();
+	void ResetDefaultColors();
+	QColor GetColor(uint32_t colorVal, QString label);
+	uint32_t preset = 0;
+	uint32_t selectRed = 0x0000FF;
+	uint32_t selectGreen = 0x00FF00;
+	uint32_t selectBlue = 0xFF7F00;
+	uint32_t mixerGreen = 0x267f26;
+	uint32_t mixerYellow = 0x267f7f;
+	uint32_t mixerRed = 0x26267f;
+	uint32_t mixerGreenActive = 0x4cff4c;
+	uint32_t mixerYellowActive = 0x4cffff;
+	uint32_t mixerRedActive = 0x4c4cff;
+
 	void SaveGeneralSettings();
 	void SaveStream1Settings();
 	void SaveOutputSettings();
 	void SaveAudioSettings();
 	void SaveVideoSettings();
 	void SaveHotkeySettings();
+	void SaveA11ySettings();
 	void SaveAdvancedSettings();
 	void SaveSettings();
 
@@ -314,7 +344,6 @@ private:
 	void UpdateAdvOutStreamDelayEstimate();
 
 	void FillSimpleRecordingValues();
-	void FillSimpleStreamingValues();
 	void FillAudioMonitoringDevices();
 
 	void RecalcOutputResPixels(const char *resText);
@@ -327,6 +356,7 @@ private:
 	QIcon audioIcon;
 	QIcon videoIcon;
 	QIcon hotkeysIcon;
+	QIcon accessibilityIcon;
 	QIcon advancedIcon;
 
 	QIcon GetGeneralIcon() const;
@@ -335,12 +365,18 @@ private:
 	QIcon GetAudioIcon() const;
 	QIcon GetVideoIcon() const;
 	QIcon GetHotkeysIcon() const;
+	QIcon GetAccessibilityIcon() const;
 	QIcon GetAdvancedIcon() const;
 
 	int CurrentFLVTrack();
 
 	OBSService GetStream1Service();
 
+	bool IsServiceOutputHasNetworkFeatures();
+
+	bool ServiceAndCodecCompatible();
+	bool ServiceSupportsCodecCheck();
+
 private slots:
 	void on_theme_activated(int idx);
 
@@ -359,7 +395,8 @@ private slots:
 	void on_advOutFFVEncoder_currentIndexChanged(int idx);
 	void on_advOutFFType_currentIndexChanged(int idx);
 
-	void on_colorFormat_currentIndexChanged(const QString &text);
+	void on_colorFormat_currentIndexChanged(int idx);
+	void on_colorSpace_currentIndexChanged(int idx);
 
 	void on_filenameFormatting_textEdited(const QString &text);
 	void on_outputResolution_editTextChanged(const QString &text);
@@ -367,6 +404,17 @@ private slots:
 
 	void on_disableOSXVSync_clicked();
 
+	void on_choose1_clicked();
+	void on_choose2_clicked();
+	void on_choose3_clicked();
+	void on_choose4_clicked();
+	void on_choose5_clicked();
+	void on_choose6_clicked();
+	void on_choose7_clicked();
+	void on_choose8_clicked();
+	void on_choose9_clicked();
+	void on_colorPreset_currentIndexChanged(int idx);
+
 	void GeneralChanged();
 	void HideOBSWindowWarning(int state);
 	void AudioChanged();
@@ -374,6 +422,8 @@ private slots:
 	void ReloadAudioSources();
 	void SurroundWarning(int idx);
 	void SpeakerLayoutChanged(int idx);
+	void LowLatencyBufferingChanged(bool checked);
+	void UpdateAudioWarnings();
 	void OutputsChanged();
 	void Stream1Changed();
 	void VideoChanged();
@@ -382,13 +432,17 @@ private slots:
 	void HotkeysChanged();
 	bool ScanDuplicateHotkeys(QFormLayout *layout);
 	void ReloadHotkeys(obs_hotkey_id ignoreKey = OBS_INVALID_HOTKEY_ID);
+	void A11yChanged();
 	void AdvancedChanged();
 	void AdvancedChangedRestart();
 
 	void UpdateStreamDelayEstimate();
 
+	void UpdateAdvNetworkGroup();
+
 	void UpdateAutomaticReplayBufferCheckboxes();
 
+	void AdvOutSplitFileChanged();
 	void AdvOutRecCheckWarnings();
 
 	void SimpleRecordingQualityChanged();
@@ -408,6 +462,7 @@ private slots:
 	void SetAudioIcon(const QIcon &icon);
 	void SetVideoIcon(const QIcon &icon);
 	void SetHotkeysIcon(const QIcon &icon);
+	void SetAccessibilityIcon(const QIcon &icon);
 	void SetAdvancedIcon(const QIcon &icon);
 
 	void UseStreamKeyAdvClicked();
