@@ -16,7 +16,6 @@
 ******************************************************************************/
 
 #include "gl-nix.h"
-#include "gl-x11-glx.h"
 #include "gl-x11-egl.h"
 
 #ifdef ENABLE_WAYLAND
@@ -30,9 +29,6 @@ static void init_winsys(void)
 	assert(gl_vtable == NULL);
 
 	switch (obs_get_nix_platform()) {
-	case OBS_NIX_PLATFORM_X11_GLX:
-		gl_vtable = gl_x11_glx_get_winsys_vtable();
-		break;
 	case OBS_NIX_PLATFORM_X11_EGL:
 		gl_vtable = gl_x11_egl_get_winsys_vtable();
 		break;
@@ -119,11 +115,25 @@ extern void device_load_swapchain(gs_device_t *device, gs_swapchain_t *swap)
 	gl_vtable->device_load_swapchain(device, swap);
 }
 
+extern bool device_is_present_ready(gs_device_t *device)
+{
+	UNUSED_PARAMETER(device);
+	return true;
+}
+
 extern void device_present(gs_device_t *device)
 {
 	gl_vtable->device_present(device);
 }
 
+extern bool device_is_monitor_hdr(gs_device_t *device, void *monitor)
+{
+	UNUSED_PARAMETER(device);
+	UNUSED_PARAMETER(monitor);
+
+	return false;
+}
+
 extern struct gs_texture *device_texture_create_from_dmabuf(
 	gs_device_t *device, unsigned int width, unsigned int height,
 	uint32_t drm_format, enum gs_color_format color_format,
@@ -152,3 +162,11 @@ extern bool device_query_dmabuf_modifiers_for_format(gs_device_t *device,
 	return gl_vtable->device_query_dmabuf_modifiers_for_format(
 		device, drm_format, modifiers, n_modifiers);
 }
+
+struct gs_texture *device_texture_create_from_pixmap(
+	gs_device_t *device, uint32_t width, uint32_t height,
+	enum gs_color_format color_format, uint32_t target, void *pixmap)
+{
+	return gl_vtable->device_texture_create_from_pixmap(
+		device, width, height, color_format, target, pixmap);
+}
