@@ -26,6 +26,7 @@
 #include "platform.h"
 #include "darray.h"
 #include "dstr.h"
+#include "obsconfig.h"
 #include "util_uint64.h"
 #include "windows/win-registry.h"
 #include "windows/win-version.h"
@@ -136,7 +137,145 @@ void os_dlclose(void *module)
 	FreeLibrary(module);
 }
 
-bool os_is_obs_plugin(const char *path)
+#if OBS_QT_VERSION == 6
+static bool has_qt5_import(VOID *base, PIMAGE_NT_HEADERS nt_headers)
+{
+	__try {
+		PIMAGE_DATA_DIRECTORY data_dir;
+		data_dir =
+			&nt_headers->OptionalHeader
+				 .DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT];
+
+		if (data_dir->Size == 0)
+			return false;
+
+		PIMAGE_SECTION_HEADER section, last_section;
+		section = IMAGE_FIRST_SECTION(nt_headers);
+		last_section = section;
+
+		/* find the section that contains the export directory */
+		int i;
+		for (i = 0; i < nt_headers->FileHeader.NumberOfSections; i++) {
+			if (section->VirtualAddress <=
+			    data_dir->VirtualAddress) {
+				last_section = section;
+				section++;
+				continue;
+			} else {
+				break;
+			}
+		}
+
+		/* double check in case we exited early */
+		if (last_section->VirtualAddress > data_dir->VirtualAddress ||
+		    section->VirtualAddress <= data_dir->VirtualAddress)
+			return false;
+
+		section = last_section;
+
+		/* get a pointer to the import directory */
+		PIMAGE_IMPORT_DESCRIPTOR import;
+		import = (PIMAGE_IMPORT_DESCRIPTOR)((byte *)base +
+						    data_dir->VirtualAddress -
+						    section->VirtualAddress +
+						    section->PointerToRawData);
+
+		while (import->Name != 0) {
+			char *name = (char *)((byte *)base + import->Name -
+					      section->VirtualAddress +
+					      section->PointerToRawData);
+
+			/* qt5? bingo, reject this library */
+			if (astrcmpi_n(name, "qt5", 3) == 0) {
+				return true;
+			}
+
+			import++;
+		}
+
+	} __except (EXCEPTION_EXECUTE_HANDLER) {
+		/* we failed somehow, for compatibility assume no qt5 import */
+		return false;
+	}
+
+	return false;
+}
+#endif
+
+static bool has_obs_export(VOID *base, PIMAGE_NT_HEADERS nt_headers)
+{
+	__try {
+		PIMAGE_DATA_DIRECTORY data_dir;
+		data_dir =
+			&nt_headers->OptionalHeader
+				 .DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT];
+
+		if (data_dir->Size == 0)
+			return false;
+
+		PIMAGE_SECTION_HEADER section, last_section;
+		section = IMAGE_FIRST_SECTION(nt_headers);
+		last_section = section;
+
+		/* find the section that contains the export directory */
+		int i;
+		for (i = 0; i < nt_headers->FileHeader.NumberOfSections; i++) {
+			if (section->VirtualAddress <=
+			    data_dir->VirtualAddress) {
+				last_section = section;
+				section++;
+				continue;
+			} else {
+				break;
+			}
+		}
+
+		/* double check in case we exited early */
+		if (last_section->VirtualAddress > data_dir->VirtualAddress ||
+		    section->VirtualAddress <= data_dir->VirtualAddress)
+			return false;
+
+		section = last_section;
+
+		/* get a pointer to the export directory */
+		PIMAGE_EXPORT_DIRECTORY export;
+		export = (PIMAGE_EXPORT_DIRECTORY)((byte *)base +
+						   data_dir->VirtualAddress -
+						   section->VirtualAddress +
+						   section->PointerToRawData);
+
+		if (export->NumberOfNames == 0)
+			return false;
+
+		/* get a pointer to the export directory names */
+		DWORD *names_ptr;
+		names_ptr = (DWORD *)((byte *)base + export->AddressOfNames -
+				      section->VirtualAddress +
+				      section->PointerToRawData);
+
+		/* iterate through each name and see if its an obs plugin */
+		CHAR *name;
+		size_t j;
+		for (j = 0; j < export->NumberOfNames; j++) {
+
+			name = (CHAR *)base + names_ptr[j] -
+			       section->VirtualAddress +
+			       section->PointerToRawData;
+
+			if (!strcmp(name, "obs_module_load")) {
+				return true;
+			}
+		}
+	} __except (EXCEPTION_EXECUTE_HANDLER) {
+		/* we failed somehow, for compatibility let's assume it
+		 * was a valid plugin and let the loader deal with it */
+		return true;
+	}
+
+	return false;
+}
+
+void get_plugin_info(const char *path, bool *is_obs_plugin, bool *can_load)
 {
 	struct dstr dll_name;
 	wchar_t *wpath;
@@ -147,12 +286,12 @@ bool os_is_obs_plugin(const char *path)
 
 	PIMAGE_DOS_HEADER dos_header;
 	PIMAGE_NT_HEADERS nt_headers;
-	PIMAGE_SECTION_HEADER section, last_section;
 
-	bool ret = false;
+	*is_obs_plugin = false;
+	*can_load = false;
 
 	if (!path)
-		return false;
+		return;
 
 	dstr_init_copy(&dll_name, path);
 	dstr_replace(&dll_name, "\\", "/");
@@ -193,72 +332,21 @@ bool os_is_obs_plugin(const char *path)
 		if (nt_headers->Signature != IMAGE_NT_SIGNATURE)
 			goto cleanup;
 
-		PIMAGE_DATA_DIRECTORY data_dir;
-		data_dir =
-			&nt_headers->OptionalHeader
-				 .DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT];
+		*is_obs_plugin = has_obs_export(base, nt_headers);
 
-		if (data_dir->Size == 0)
-			goto cleanup;
-
-		section = IMAGE_FIRST_SECTION(nt_headers);
-		last_section = section;
-
-		/* find the section that contains the export directory */
-		int i;
-		for (i = 0; i < nt_headers->FileHeader.NumberOfSections; i++) {
-			if (section->VirtualAddress <=
-			    data_dir->VirtualAddress) {
-				last_section = section;
-				section++;
-				continue;
-			} else {
-				break;
-			}
-		}
-
-		/* double check in case we exited early */
-		if (last_section->VirtualAddress > data_dir->VirtualAddress ||
-		    section->VirtualAddress <= data_dir->VirtualAddress)
-			goto cleanup;
-
-		section = last_section;
-
-		/* get a pointer to the export directory */
-		PIMAGE_EXPORT_DIRECTORY export;
-		export = (PIMAGE_EXPORT_DIRECTORY)((byte *)base +
-						   data_dir->VirtualAddress -
-						   section->VirtualAddress +
-						   section->PointerToRawData);
-
-		if (export->NumberOfNames == 0)
-			goto cleanup;
-
-		/* get a pointer to the export directory names */
-		DWORD *names_ptr;
-		names_ptr = (DWORD *)((byte *)base + export->AddressOfNames -
-				      section->VirtualAddress +
-				      section->PointerToRawData);
-
-		/* iterate through each name and see if its an obs plugin */
-		CHAR *name;
-		size_t j;
-		for (j = 0; j < export->NumberOfNames; j++) {
-
-			name = (CHAR *)base + names_ptr[j] -
-			       section->VirtualAddress +
-			       section->PointerToRawData;
-
-			if (!strcmp(name, "obs_module_load")) {
-				ret = true;
-				goto cleanup;
-			}
+#if OBS_QT_VERSION == 6
+		if (*is_obs_plugin) {
+			*can_load = !has_qt5_import(base, nt_headers);
 		}
+#else
+		*can_load = true;
+#endif
 
 	} __except (EXCEPTION_EXECUTE_HANDLER) {
 		/* we failed somehow, for compatibility let's assume it
 		 * was a valid plugin and let the loader deal with it */
-		ret = true;
+		*is_obs_plugin = true;
+		*can_load = true;
 		goto cleanup;
 	}
 
@@ -271,8 +359,16 @@ cleanup:
 
 	if (hFile != INVALID_HANDLE_VALUE)
 		CloseHandle(hFile);
+}
 
-	return ret;
+bool os_is_obs_plugin(const char *path)
+{
+	bool is_obs_plugin;
+	bool can_load;
+
+	get_plugin_info(path, &is_obs_plugin, &can_load);
+
+	return is_obs_plugin && can_load;
 }
 
 union time_data {
@@ -360,6 +456,24 @@ bool os_sleepto_ns(uint64_t time_target)
 	return stall;
 }
 
+bool os_sleepto_ns_fast(uint64_t time_target)
+{
+	uint64_t current = os_gettime_ns();
+	if (time_target < current)
+		return false;
+
+	do {
+		uint64_t remain_ms = (time_target - current) / 1000000;
+		if (!remain_ms)
+			remain_ms = 1;
+		Sleep((DWORD)remain_ms);
+
+		current = os_gettime_ns();
+	} while (time_target > current);
+
+	return true;
+}
+
 void os_sleep_ms(uint32_t duration)
 {
 	/* windows 8+ appears to have decreased sleep precision */
@@ -604,11 +718,13 @@ static void make_globent(struct os_globent *ent, WIN32_FIND_DATA *wfd,
 
 	dstr_from_wcs(&name, wfd->cFileName);
 	dstr_copy(&path, pattern);
-	slash = strrchr(path.array, '/');
-	if (slash)
-		dstr_resize(&path, slash + 1 - path.array);
-	else
-		dstr_free(&path);
+	if (path.array) {
+		slash = strrchr(path.array, '/');
+		if (slash)
+			dstr_resize(&path, slash + 1 - path.array);
+		else
+			dstr_free(&path);
+	}
 
 	dstr_cat_dstr(&path, &name);
 	ent->path = path.array;
@@ -845,7 +961,7 @@ char *os_getcwd(char *path, size_t size)
 	if (!len)
 		return NULL;
 
-	path_w = bmalloc((len + 1) * sizeof(wchar_t));
+	path_w = bmalloc(((size_t)len + 1) * sizeof(wchar_t));
 	GetCurrentDirectoryW(len + 1, path_w);
 	os_wcs_to_utf8(path_w, (size_t)len, path, size);
 	bfree(path_w);
@@ -972,6 +1088,11 @@ bool is_64_bit_windows(void)
 #endif
 }
 
+bool os_get_emulation_status(void)
+{
+	return false;
+}
+
 void get_reg_dword(HKEY hkey, LPCWSTR sub_key, LPCWSTR value_name,
 		   struct reg_dword *info)
 {
@@ -1002,9 +1123,9 @@ void get_reg_dword(HKEY hkey, LPCWSTR sub_key, LPCWSTR value_name,
 
 static inline void rtl_get_ver(struct win_version_info *ver)
 {
-	RTL_OSVERSIONINFOEXW osver = {0};
 	HMODULE ntdll = GetModuleHandleW(L"ntdll");
-	NTSTATUS s;
+	if (!ntdll)
+		return;
 
 	NTSTATUS(WINAPI * get_ver)
 	(RTL_OSVERSIONINFOEXW *) =
@@ -1013,8 +1134,9 @@ static inline void rtl_get_ver(struct win_version_info *ver)
 		return;
 	}
 
+	RTL_OSVERSIONINFOEXW osver = {0};
 	osver.dwOSVersionInfoSize = sizeof(osver);
-	s = get_ver(&osver);
+	NTSTATUS s = get_ver(&osver);
 	if (s < 0) {
 		return;
 	}
@@ -1026,13 +1148,10 @@ static inline void rtl_get_ver(struct win_version_info *ver)
 }
 
 static inline bool get_reg_sz(HKEY key, const wchar_t *val, wchar_t *buf,
-			      const size_t size)
+			      DWORD size)
 {
-	DWORD dwsize = (DWORD)size;
-	LSTATUS status;
-
-	status = RegQueryValueExW(key, val, NULL, NULL, (LPBYTE)buf, &dwsize);
-	buf[(size / sizeof(wchar_t)) - 1] = 0;
+	const LSTATUS status =
+		RegGetValueW(key, NULL, val, RRF_RT_REG_SZ, NULL, buf, &size);
 	return status == ERROR_SUCCESS;
 }
 
@@ -1219,23 +1338,27 @@ static void os_get_cores_internal(void)
 
 	info = malloc(len);
 
-	if (GetLogicalProcessorInformation(info, &len)) {
-		DWORD num = len / sizeof(*info);
-		temp = info;
+	if (info) {
+		if (GetLogicalProcessorInformation(info, &len)) {
+			DWORD num = len / sizeof(*info);
+			temp = info;
 
-		for (DWORD i = 0; i < num; i++) {
-			if (temp->Relationship == RelationProcessorCore) {
-				ULONG_PTR mask = temp->ProcessorMask;
+			for (DWORD i = 0; i < num; i++) {
+				if (temp->Relationship ==
+				    RelationProcessorCore) {
+					ULONG_PTR mask = temp->ProcessorMask;
 
-				physical_cores++;
-				logical_cores += num_logical_cores(mask);
+					physical_cores++;
+					logical_cores +=
+						num_logical_cores(mask);
+				}
+
+				temp++;
 			}
-
-			temp++;
 		}
-	}
 
-	free(info);
+		free(info);
+	}
 }
 
 int os_get_physical_cores(void)
