@@ -142,14 +142,14 @@ void DeckLinkDeviceInstance::HandleVideoFrame(
 	if (videoFrame == nullptr)
 		return;
 
-	IDeckLinkVideoFrameAncillaryPackets *packets;
+	ComPtr<IDeckLinkVideoFrameAncillaryPackets> packets;
 
 	if (videoFrame->QueryInterface(IID_IDeckLinkVideoFrameAncillaryPackets,
 				       (void **)&packets) == S_OK) {
-		IDeckLinkAncillaryPacketIterator *iterator;
+		ComPtr<IDeckLinkAncillaryPacketIterator> iterator;
 		packets->GetPacketIterator(&iterator);
 
-		IDeckLinkAncillaryPacket *packet;
+		ComPtr<IDeckLinkAncillaryPacket> packet;
 		iterator->Next(&packet);
 
 		if (packet) {
@@ -160,18 +160,13 @@ void DeckLinkDeviceInstance::HandleVideoFrame(
 			if (did == 0x61 && sdid == 0x01) {
 				this->HandleCaptionPacket(packet, timestamp);
 			}
-
-			packet->Release();
 		}
-
-		iterator->Release();
-		packets->Release();
 	}
 
-	IDeckLinkVideoFrame *frame;
+	ComPtr<IDeckLinkVideoFrame> frame;
 	if (videoFrame->GetPixelFormat() != convertFrame->GetPixelFormat()) {
-		IDeckLinkVideoConversion *frameConverter =
-			CreateVideoConversionInstance();
+		ComPtr<IDeckLinkVideoConversion> frameConverter;
+		frameConverter.Set(CreateVideoConversionInstance());
 
 		frameConverter->ConvertFrame(videoFrame, convertFrame);
 
@@ -192,6 +187,9 @@ void DeckLinkDeviceInstance::HandleVideoFrame(
 	currentFrame.height = (uint32_t)frame->GetHeight();
 	currentFrame.timestamp = timestamp;
 
+	if (currentFrame.width == 0 || currentFrame.height == 0)
+		return;
+
 	obs_source_output_video2(
 		static_cast<DeckLinkInput *>(decklink)->GetSource(),
 		&currentFrame);
@@ -310,7 +308,8 @@ void DeckLinkDeviceInstance::SetupVideoFormat(DeckLinkDeviceMode *mode_)
 	if (mode_ == nullptr)
 		return;
 
-	currentFrame.format = ConvertPixelFormat(pixelFormat);
+	const enum video_format format = ConvertPixelFormat(pixelFormat);
+	currentFrame.format = format;
 
 	colorSpace = static_cast<DeckLinkInput *>(decklink)->GetColorSpace();
 	if (colorSpace == VIDEO_CS_DEFAULT) {
@@ -328,10 +327,9 @@ void DeckLinkDeviceInstance::SetupVideoFormat(DeckLinkDeviceMode *mode_)
 	colorRange = static_cast<DeckLinkInput *>(decklink)->GetColorRange();
 	currentFrame.range = colorRange;
 
-	video_format_get_parameters(activeColorSpace, colorRange,
-				    currentFrame.color_matrix,
-				    currentFrame.color_range_min,
-				    currentFrame.color_range_max);
+	video_format_get_parameters_for_format(
+		activeColorSpace, colorRange, format, currentFrame.color_matrix,
+		currentFrame.color_range_min, currentFrame.color_range_max);
 
 	delete convertFrame;
 
@@ -373,7 +371,6 @@ bool DeckLinkDeviceInstance::StartCapture(DeckLinkDeviceMode *mode_,
 	if (!device->GetInput(&input))
 		return false;
 
-	IDeckLinkConfiguration *deckLinkConfiguration = NULL;
 	HRESULT result = input->QueryInterface(IID_IDeckLinkConfiguration,
 					       (void **)&deckLinkConfiguration);
 	if (result != S_OK) {
@@ -526,7 +523,7 @@ bool DeckLinkDeviceInstance::StartOutput(DeckLinkDeviceMode *mode_)
 
 	int keyerMode = device->GetKeyerMode();
 
-	IDeckLinkKeyer *deckLinkKeyer = nullptr;
+	ComPtr<IDeckLinkKeyer> deckLinkKeyer;
 	if (device->GetKeyer(&deckLinkKeyer)) {
 		if (keyerMode) {
 			deckLinkKeyer->Enable(keyerMode == 1);
@@ -574,10 +571,8 @@ bool DeckLinkDeviceInstance::StopOutput()
 	output->DisableVideoOutput();
 	output->DisableAudioOutput();
 
-	if (decklinkOutputFrame != nullptr) {
-		decklinkOutputFrame->Release();
+	if (decklinkOutputFrame != nullptr)
 		decklinkOutputFrame = nullptr;
-	}
 
 	return true;
 }
