@@ -37,7 +37,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "v4l2-controls.h"
 #include "v4l2-helpers.h"
-#include "v4l2-mjpeg.h"
+#include "v4l2-decoder.h"
 
 #if HAVE_UDEV
 #include "v4l2-udev.h"
@@ -81,7 +81,7 @@ struct v4l2_data {
 	obs_source_t *source;
 	pthread_t thread;
 	os_event_t *event;
-	struct v4l2_mjpeg_decoder mjpeg_decoder;
+	struct v4l2_decoder decoder;
 
 	bool framerate_unchanged;
 	bool resolution_unchanged;
@@ -121,12 +121,16 @@ static void v4l2_prep_obs_frame(struct v4l2_data *data,
 	memset(frame, 0, sizeof(struct obs_source_frame));
 	memset(plane_offsets, 0, sizeof(size_t) * MAX_AV_PLANES);
 
+	const enum video_format format = v4l2_to_obs_video_format(data->pixfmt);
+
 	frame->width = data->width;
 	frame->height = data->height;
-	frame->format = v4l2_to_obs_video_format(data->pixfmt);
-	video_format_get_parameters(VIDEO_CS_DEFAULT, data->color_range,
-				    frame->color_matrix, frame->color_range_min,
-				    frame->color_range_max);
+	frame->format = format;
+	video_format_get_parameters_for_format(VIDEO_CS_DEFAULT,
+					       data->color_range, format,
+					       frame->color_matrix,
+					       frame->color_range_min,
+					       frame->color_range_max);
 
 	switch (data->pixfmt) {
 	case V4L2_PIX_FMT_NV12:
@@ -264,10 +268,12 @@ static void *v4l2_thread(void *vptr)
 
 		start = (uint8_t *)data->buffers.info[buf.index].start;
 
-		if (data->pixfmt == V4L2_PIX_FMT_MJPEG) {
-			if (v4l2_decode_mjpeg(&out, start, buf.bytesused,
-					      &data->mjpeg_decoder) < 0) {
-				blog(LOG_ERROR, "failed to unpack jpeg");
+		if (data->pixfmt == V4L2_PIX_FMT_MJPEG ||
+		    data->pixfmt == V4L2_PIX_FMT_H264) {
+			if (v4l2_decode_frame(&out, start, buf.bytesused,
+					      &data->decoder) < 0) {
+				blog(LOG_ERROR,
+				     "failed to unpack jpeg or h264");
 				break;
 			}
 		} else {
@@ -473,7 +479,8 @@ static void v4l2_format_list(int dev, obs_property_t *prop)
 
 		if (v4l2_to_obs_video_format(fmt.pixelformat) !=
 			    VIDEO_FORMAT_NONE ||
-		    fmt.pixelformat == V4L2_PIX_FMT_MJPEG) {
+		    fmt.pixelformat == V4L2_PIX_FMT_MJPEG ||
+		    fmt.pixelformat == V4L2_PIX_FMT_H264) {
 			obs_property_list_add_int(prop, buffer.array,
 						  fmt.pixelformat);
 			blog(LOG_INFO, "Pixelformat: %s (available)",
@@ -906,7 +913,10 @@ static void v4l2_terminate(struct v4l2_data *data)
 		data->thread = 0;
 	}
 
-	v4l2_destroy_mjpeg(&data->mjpeg_decoder);
+	if (data->pixfmt == V4L2_PIX_FMT_MJPEG ||
+	    data->pixfmt == V4L2_PIX_FMT_H264) {
+		v4l2_destroy_decoder(&data->decoder);
+	}
 	v4l2_destroy_mmap(&data->buffers);
 
 	if (data->dev != -1) {
@@ -998,7 +1008,8 @@ static void v4l2_init(struct v4l2_data *data)
 		goto fail;
 	}
 	if (v4l2_to_obs_video_format(data->pixfmt) == VIDEO_FORMAT_NONE &&
-	    data->pixfmt != V4L2_PIX_FMT_MJPEG) {
+	    data->pixfmt != V4L2_PIX_FMT_MJPEG &&
+	    data->pixfmt != V4L2_PIX_FMT_H264) {
 		blog(LOG_ERROR, "Selected video format not supported");
 		goto fail;
 	}
@@ -1021,9 +1032,12 @@ static void v4l2_init(struct v4l2_data *data)
 		goto fail;
 	}
 
-	if (v4l2_init_mjpeg(&data->mjpeg_decoder) < 0) {
-		blog(LOG_ERROR, "Failed to initialize mjpeg decoder");
-		goto fail;
+	if (data->pixfmt == V4L2_PIX_FMT_MJPEG ||
+	    data->pixfmt == V4L2_PIX_FMT_H264) {
+		if (v4l2_init_decoder(&data->decoder, data->pixfmt) < 0) {
+			blog(LOG_ERROR, "Failed to initialize decoder");
+			goto fail;
+		}
 	}
 
 	/* start the capture thread */
@@ -1033,7 +1047,7 @@ static void v4l2_init(struct v4l2_data *data)
 		goto fail;
 	return;
 fail:
-	blog(LOG_ERROR, "Initialization failed");
+	blog(LOG_ERROR, "Initialization failed, errno: %s", strerror(errno));
 	v4l2_terminate(data);
 }
 
@@ -1162,6 +1176,8 @@ static void *v4l2_create(obs_data_t *settings, obs_source_t *source)
 
 	signal_handler_connect(sh, "device_added", &device_added, data);
 	signal_handler_connect(sh, "device_removed", &device_removed, data);
+#else
+	blog(LOG_INFO, "Compiled without libudev, you can't reconnect devices");
 #endif
 
 	return data;
