@@ -26,8 +26,14 @@
 #include <stdlib.h>
 #include "ffmpeg-mux.h"
 
+#include <util/threading.h>
+#include <util/platform.h>
+#include <util/circlebuf.h>
 #include <util/dstr.h>
+#include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
+#include <libavutil/channel_layout.h>
+#include <libavutil/mastering_display_metadata.h>
 
 #define ANSI_COLOR_RED "\x1b[0;91m"
 #define ANSI_COLOR_MAGENTA "\x1b[0;95m"
@@ -39,6 +45,8 @@
 #define CODEC_FLAG_GLOBAL_H CODEC_FLAG_GLOBAL_HEADER
 #endif
 
+#define AVIO_BUFFER_SIZE 65536
+
 /* ------------------------------------------------------------------------- */
 
 static char *global_stream_key = "";
@@ -91,6 +99,7 @@ struct main_params {
 	int color_trc;
 	int colorspace;
 	int color_range;
+	int max_luminance;
 	char *acodec;
 	char *muxer_settings;
 };
@@ -99,6 +108,7 @@ struct audio_params {
 	char *name;
 	int abitrate;
 	int sample_rate;
+	int frame_size;
 	int channels;
 };
 
@@ -112,10 +122,29 @@ struct audio_info {
 	AVCodecContext *ctx;
 };
 
+struct io_header {
+	uint64_t seek_offset;
+	size_t data_length;
+};
+
+struct io_buffer {
+	bool active;
+	bool shutdown_requested;
+	bool output_error;
+	os_event_t *buffer_space_available_event;
+	os_event_t *new_data_available_event;
+	pthread_t io_thread;
+	pthread_mutex_t data_mutex;
+	FILE *output_file;
+	struct circlebuf data;
+	uint64_t next_pos;
+};
+
 struct ffmpeg_mux {
 	AVFormatContext *output;
 	AVStream *video_stream;
 	AVCodecContext *video_ctx;
+	AVPacket *packet;
 	struct audio_info *audio_infos;
 	struct main_params params;
 	struct audio_params *audio;
@@ -123,7 +152,7 @@ struct ffmpeg_mux {
 	struct header *audio_header;
 	int num_audio_streams;
 	bool initialized;
-	char error[4096];
+	struct io_buffer io;
 };
 
 static void header_free(struct header *header)
@@ -134,9 +163,7 @@ static void header_free(struct header *header)
 static void free_avformat(struct ffmpeg_mux *ffm)
 {
 	if (ffm->output) {
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
 		avcodec_free_context(&ffm->video_ctx);
-#endif
 
 		if ((ffm->output->oformat->flags & AVFMT_NOFILE) == 0)
 			avio_close(ffm->output->pb);
@@ -146,10 +173,9 @@ static void free_avformat(struct ffmpeg_mux *ffm)
 	}
 
 	if (ffm->audio_infos) {
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
 		for (int i = 0; i < ffm->num_audio_streams; ++i)
 			avcodec_free_context(&ffm->audio_infos[i].ctx);
-#endif
+
 		free(ffm->audio_infos);
 	}
 
@@ -164,6 +190,29 @@ static void ffmpeg_mux_free(struct ffmpeg_mux *ffm)
 		av_write_trailer(ffm->output);
 	}
 
+	// If we're writing to a file with the circlebuf, shut it
+	// down gracefully
+	if (ffm->io.active) {
+		os_atomic_set_bool(&ffm->io.shutdown_requested, true);
+
+		// Wakes up the I/O thread and waits for it to finish
+		pthread_mutex_lock(&ffm->io.data_mutex);
+		os_event_signal(ffm->io.new_data_available_event);
+		pthread_mutex_unlock(&ffm->io.data_mutex);
+		pthread_join(ffm->io.io_thread, NULL);
+
+		// Cleanup everything else
+		av_free(ffm->output->pb->buffer);
+		avio_context_free(&ffm->output->pb);
+
+		os_event_destroy(ffm->io.new_data_available_event);
+		os_event_destroy(ffm->io.buffer_space_available_event);
+
+		pthread_mutex_destroy(&ffm->io.data_mutex);
+
+		circlebuf_free(&ffm->io.data);
+	}
+
 	free_avformat(ffm);
 
 	header_free(&ffm->video_header);
@@ -182,6 +231,8 @@ static void ffmpeg_mux_free(struct ffmpeg_mux *ffm)
 
 	dstr_free(&ffm->params.printable_file);
 
+	av_packet_free(&ffm->packet);
+
 	memset(ffm, 0, sizeof(*ffm));
 }
 
@@ -223,6 +274,8 @@ static bool get_audio_params(struct audio_params *audio, int *argc,
 		return false;
 	if (!get_opt_int(argc, argv, &audio->sample_rate, "audio sample rate"))
 		return false;
+	if (!get_opt_int(argc, argv, &audio->frame_size, "audio frame size"))
+		return false;
 	if (!get_opt_int(argc, argv, &audio->channels, "audio channels"))
 		return false;
 	return true;
@@ -231,13 +284,15 @@ static bool get_audio_params(struct audio_params *audio, int *argc,
 static void ffmpeg_log_callback(void *param, int level, const char *format,
 				va_list args)
 {
-#ifdef DEBUG_FFMPEG
+#ifdef ENABLE_FFMPEG_MUX_DEBUG
 	char out_buffer[4096];
 	struct dstr out = {0};
 
 	vsnprintf(out_buffer, sizeof(out_buffer), format, args);
 	dstr_copy(&out, out_buffer);
-	dstr_replace(&out, global_stream_key, "{stream_key}");
+	if (global_stream_key && *global_stream_key) {
+		dstr_replace(&out, global_stream_key, "{stream_key}");
+	}
 
 	switch (level) {
 	case AV_LOG_INFO:
@@ -313,6 +368,9 @@ static bool init_params(int *argc, char ***argv, struct main_params *params,
 		if (!get_opt_int(argc, argv, &params->color_range,
 				 "video color range"))
 			return false;
+		if (!get_opt_int(argc, argv, &params->max_luminance,
+				 "video max luminance"))
+			return false;
 		if (!get_opt_int(argc, argv, &params->fps_num, "video fps num"))
 			return false;
 		if (!get_opt_int(argc, argv, &params->fps_den, "video fps den"))
@@ -351,22 +409,9 @@ static bool init_params(int *argc, char ***argv, struct main_params *params,
 }
 
 static bool new_stream(struct ffmpeg_mux *ffm, AVStream **stream,
-		       const char *name, AVCodec **codec)
+		       const char *name)
 {
-	const AVCodecDescriptor *desc = avcodec_descriptor_get_by_name(name);
-
-	if (!desc) {
-		fprintf(stderr, "Couldn't find encoder '%s'\n", name);
-		return false;
-	}
-
-	*codec = avcodec_find_encoder(desc->id);
-	if (!*codec) {
-		fprintf(stderr, "Couldn't create encoder\n");
-		return false;
-	}
-
-	*stream = avformat_new_stream(ffm->output, *codec);
+	*stream = avformat_new_stream(ffm->output, NULL);
 	if (!*stream) {
 		fprintf(stderr, "Couldn't create stream for encoder '%s'\n",
 			name);
@@ -379,11 +424,17 @@ static bool new_stream(struct ffmpeg_mux *ffm, AVStream **stream,
 
 static void create_video_stream(struct ffmpeg_mux *ffm)
 {
-	AVCodec *codec;
 	AVCodecContext *context;
 	void *extradata = NULL;
+	const char *name = ffm->params.vcodec;
 
-	if (!new_stream(ffm, &ffm->video_stream, ffm->params.vcodec, &codec))
+	const AVCodecDescriptor *codec = avcodec_descriptor_get_by_name(name);
+	if (!codec) {
+		fprintf(stderr, "Couldn't find codec '%s'\n", name);
+		return;
+	}
+
+	if (!new_stream(ffm, &ffm->video_stream, name))
 		return;
 
 	if (ffm->video_header.size) {
@@ -391,11 +442,9 @@ static void create_video_stream(struct ffmpeg_mux *ffm)
 				      ffm->video_header.size);
 	}
 
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
-	context = avcodec_alloc_context3(codec);
-#else
-	context = ffm->video_stream->codec;
-#endif
+	context = avcodec_alloc_context3(NULL);
+	context->codec_type = codec->type;
+	context->codec_id = codec->id;
 	context->bit_rate = (int64_t)ffm->params.vbitrate * 1000;
 	context->width = ffm->params.width;
 	context->height = ffm->params.height;
@@ -405,6 +454,10 @@ static void create_video_stream(struct ffmpeg_mux *ffm)
 	context->color_trc = ffm->params.color_trc;
 	context->colorspace = ffm->params.colorspace;
 	context->color_range = ffm->params.color_range;
+	context->chroma_sample_location =
+		(ffm->params.colorspace == AVCOL_SPC_BT2020_NCL)
+			? AVCHROMA_LOC_TOPLEFT
+			: AVCHROMA_LOC_LEFT;
 	context->extradata = extradata;
 	context->extradata_size = ffm->video_header.size;
 	context->time_base =
@@ -417,24 +470,59 @@ static void create_video_stream(struct ffmpeg_mux *ffm)
 #endif
 	ffm->video_stream->avg_frame_rate = av_inv_q(context->time_base);
 
+	const int max_luminance = ffm->params.max_luminance;
+	if (max_luminance > 0) {
+		size_t content_size;
+		AVContentLightMetadata *const content =
+			av_content_light_metadata_alloc(&content_size);
+		content->MaxCLL = max_luminance;
+		content->MaxFALL = max_luminance;
+		av_stream_add_side_data(ffm->video_stream,
+					AV_PKT_DATA_CONTENT_LIGHT_LEVEL,
+					(uint8_t *)content, content_size);
+
+		AVMasteringDisplayMetadata *const mastering =
+			av_mastering_display_metadata_alloc();
+		mastering->display_primaries[0][0] = av_make_q(17, 25);
+		mastering->display_primaries[0][1] = av_make_q(8, 25);
+		mastering->display_primaries[1][0] = av_make_q(53, 200);
+		mastering->display_primaries[1][1] = av_make_q(69, 100);
+		mastering->display_primaries[2][0] = av_make_q(3, 20);
+		mastering->display_primaries[2][1] = av_make_q(3, 50);
+		mastering->white_point[0] = av_make_q(3127, 10000);
+		mastering->white_point[1] = av_make_q(329, 1000);
+		mastering->min_luminance = av_make_q(0, 1);
+		mastering->max_luminance = av_make_q(max_luminance, 1);
+		mastering->has_primaries = 1;
+		mastering->has_luminance = 1;
+		av_stream_add_side_data(ffm->video_stream,
+					AV_PKT_DATA_MASTERING_DISPLAY_METADATA,
+					(uint8_t *)mastering,
+					sizeof(*mastering));
+	}
+
 	if (ffm->output->oformat->flags & AVFMT_GLOBALHEADER)
 		context->flags |= CODEC_FLAG_GLOBAL_H;
 
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
 	avcodec_parameters_from_context(ffm->video_stream->codecpar, context);
-#endif
 
 	ffm->video_ctx = context;
 }
 
 static void create_audio_stream(struct ffmpeg_mux *ffm, int idx)
 {
-	AVCodec *codec;
 	AVCodecContext *context;
 	AVStream *stream;
 	void *extradata = NULL;
+	const char *name = ffm->params.acodec;
 
-	if (!new_stream(ffm, &stream, ffm->params.acodec, &codec))
+	const AVCodecDescriptor *codec = avcodec_descriptor_get_by_name(name);
+	if (!codec) {
+		fprintf(stderr, "Couldn't find codec '%s'\n", name);
+		return;
+	}
+
+	if (!new_stream(ffm, &stream, name))
 		return;
 
 	av_dict_set(&stream->metadata, "title", ffm->audio[idx].name, 0);
@@ -446,32 +534,33 @@ static void create_audio_stream(struct ffmpeg_mux *ffm, int idx)
 				      ffm->audio_header[idx].size);
 	}
 
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
-	context = avcodec_alloc_context3(codec);
-#else
-	context = stream->codec;
-#endif
+	context = avcodec_alloc_context3(NULL);
+	context->codec_type = codec->type;
+	context->codec_id = codec->id;
 	context->bit_rate = (int64_t)ffm->audio[idx].abitrate * 1000;
 	context->channels = ffm->audio[idx].channels;
 	context->sample_rate = ffm->audio[idx].sample_rate;
+	context->frame_size = ffm->audio[idx].frame_size;
 	context->sample_fmt = AV_SAMPLE_FMT_S16;
 	context->time_base = stream->time_base;
 	context->extradata = extradata;
 	context->extradata_size = ffm->audio_header[idx].size;
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(59, 24, 100)
 	context->channel_layout =
 		av_get_default_channel_layout(context->channels);
-	//AVlib default channel layout for 4 channels is 4.0 ; fix for quad
-	if (context->channels == 4)
-		context->channel_layout = av_get_channel_layout("quad");
-	//AVlib default channel layout for 5 channels is 5.0 ; fix for 4.1
+	//avutil default channel layout for 5 channels is 5.0 ; fix for 4.1
 	if (context->channels == 5)
 		context->channel_layout = av_get_channel_layout("4.1");
+#else
+	av_channel_layout_default(&context->ch_layout, context->channels);
+	//avutil default channel layout for 5 channels is 5.0 ; fix for 4.1
+	if (context->channels == 5)
+		context->ch_layout = (AVChannelLayout)AV_CHANNEL_LAYOUT_4POINT1;
+#endif
 	if (ffm->output->oformat->flags & AVFMT_GLOBALHEADER)
 		context->flags |= CODEC_FLAG_GLOBAL_H;
 
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
 	avcodec_parameters_from_context(stream->codecpar, context);
-#endif
 
 	ffm->audio_infos[ffm->num_audio_streams].stream = stream;
 	ffm->audio_infos[ffm->num_audio_streams].ctx = context;
@@ -573,19 +662,297 @@ static inline bool ffmpeg_mux_get_extra_data(struct ffmpeg_mux *ffm)
 #pragma warning(disable : 4996)
 #endif
 
+#define CHUNK_SIZE 1048576
+
+static void *ffmpeg_mux_io_thread(void *data)
+{
+	struct ffmpeg_mux *ffm = data;
+
+	// Chunk collects the writes into a larger batch
+	size_t chunk_used = 0;
+
+	unsigned char *chunk = malloc(CHUNK_SIZE);
+	if (!chunk) {
+		os_atomic_set_bool(&ffm->io.output_error, true);
+		fprintf(stderr, "Error allocating memory for output\n");
+		goto error;
+	}
+
+	bool shutting_down;
+	bool want_seek = false;
+	bool force_flush_chunk = false;
+
+	// current_seek_position is a virtual position updated as we read from
+	// the buffer, if it becomes discontinuous due to a seek request from
+	// ffmpeg, then we flush the chunk. next_seek_position is the actual
+	// offset we should seek to when we write the chunk.
+	uint64_t current_seek_position = 0;
+	uint64_t next_seek_position;
+
+	for (;;) {
+		// Wait for ffmpeg to write data to the buffer
+		os_event_wait(ffm->io.new_data_available_event);
+
+		// Loop to write in chunk_size chunks
+		for (;;) {
+			shutting_down = os_atomic_load_bool(
+				&ffm->io.shutdown_requested);
+
+			pthread_mutex_lock(&ffm->io.data_mutex);
+
+			// Fetch as many writes as possible from the circlebuf
+			// and fill up our local chunk. This may involve seeking
+			// if ffmpeg needs to, so take care of that as well.
+			for (;;) {
+				size_t available = ffm->io.data.size;
+
+				// Buffer is empty (now) or was already empty (we got
+				// woken up to exit)
+				if (!available)
+					break;
+
+				// Get seek offset and data size
+				struct io_header header;
+				circlebuf_peek_front(&ffm->io.data, &header,
+						     sizeof(header));
+
+				// Do we need to seek?
+				if (header.seek_offset !=
+				    current_seek_position) {
+
+					// If there's already part of a chunk pending,
+					// flush it at the current offset. Similarly,
+					// if we already plan to seek, then seek.
+					if (chunk_used || want_seek) {
+						force_flush_chunk = true;
+						break;
+					}
+
+					// Mark that we need to seek and where to
+					want_seek = true;
+					next_seek_position = header.seek_offset;
+
+					// Update our virtual position
+					current_seek_position =
+						header.seek_offset;
+				}
+
+				// Make sure there's enough room for the data, if
+				// not then force a flush
+				if (header.data_length + chunk_used >
+				    CHUNK_SIZE) {
+					force_flush_chunk = true;
+					break;
+				}
+
+				// Remove header that we already read
+				circlebuf_pop_front(&ffm->io.data, NULL,
+						    sizeof(header));
+
+				// Copy from the buffer to our local chunk
+				circlebuf_pop_front(&ffm->io.data,
+						    chunk + chunk_used,
+						    header.data_length);
+
+				// Update offsets
+				chunk_used += header.data_length;
+				current_seek_position += header.data_length;
+			}
+
+			// Signal that there is more room in the buffer
+			os_event_signal(ffm->io.buffer_space_available_event);
+
+			// Try to avoid lots of small writes unless this was the final
+			// data left in the buffer. The buffer might be entirely empty
+			// if we were woken up to exit.
+			if (!force_flush_chunk &&
+			    (!chunk_used ||
+			     (chunk_used < 65536 && !shutting_down))) {
+				os_event_reset(
+					ffm->io.new_data_available_event);
+				pthread_mutex_unlock(&ffm->io.data_mutex);
+				break;
+			}
+
+			pthread_mutex_unlock(&ffm->io.data_mutex);
+
+			// Seek if we need to
+			if (want_seek) {
+				os_fseeki64(ffm->io.output_file,
+					    next_seek_position, SEEK_SET);
+
+				// Update the next virtual position, making sure to take
+				// into account the size of the chunk we're about to write.
+				current_seek_position =
+					next_seek_position + chunk_used;
+
+				want_seek = false;
+			}
+
+			// Write the current chunk to the output file
+			if (fwrite(chunk, chunk_used, 1, ffm->io.output_file) !=
+			    1) {
+				os_atomic_set_bool(&ffm->io.output_error, true);
+				fprintf(stderr, "Error writing to '%s', %s\n",
+					ffm->params.printable_file.array,
+					strerror(errno));
+				goto error;
+			}
+
+			chunk_used = 0;
+			force_flush_chunk = false;
+		}
+
+		// If this was the last chunk, time to exit
+		if (shutting_down)
+			break;
+	}
+
+error:
+	if (chunk)
+		free(chunk);
+
+	fclose(ffm->io.output_file);
+	return NULL;
+}
+
+static int64_t ffmpeg_mux_seek_av_buffer(void *opaque, int64_t offset,
+					 int whence)
+{
+	struct ffmpeg_mux *ffm = opaque;
+
+	// If the output thread failed, signal that back up the stack
+	if (os_atomic_load_bool(&ffm->io.output_error))
+		return -1;
+
+	// Update where the next write should go
+	pthread_mutex_lock(&ffm->io.data_mutex);
+	if (whence == SEEK_SET)
+		ffm->io.next_pos = offset;
+	else if (whence == SEEK_CUR)
+		ffm->io.next_pos += offset;
+	pthread_mutex_unlock(&ffm->io.data_mutex);
+
+	return 0;
+}
+
+static int ffmpeg_mux_write_av_buffer(void *opaque, uint8_t *buf, int buf_size)
+{
+	struct ffmpeg_mux *ffm = opaque;
+
+	// If the output thread failed, signal that back up the stack
+	if (os_atomic_load_bool(&ffm->io.output_error))
+		return -1;
+
+	for (;;) {
+		pthread_mutex_lock(&ffm->io.data_mutex);
+
+		// Avoid unbounded growth of the circlebuf, cap to 256 MB
+		if (ffm->io.data.capacity >= 256 * 1048576 &&
+		    ffm->io.data.capacity - ffm->io.data.size <
+			    buf_size + sizeof(struct io_header)) {
+			// No space, wait for the I/O thread to make space
+			os_event_reset(ffm->io.buffer_space_available_event);
+			pthread_mutex_unlock(&ffm->io.data_mutex);
+			os_event_wait(ffm->io.buffer_space_available_event);
+		} else {
+			break;
+		}
+	}
+
+	struct io_header header;
+
+	header.data_length = buf_size;
+	header.seek_offset = ffm->io.next_pos;
+
+	// Copy the data into the buffer
+	circlebuf_push_back(&ffm->io.data, &header, sizeof(header));
+	circlebuf_push_back(&ffm->io.data, buf, buf_size);
+
+	// Advance the next write position
+	ffm->io.next_pos += buf_size;
+
+	// Tell the I/O thread that there's new data to be written
+	os_event_signal(ffm->io.new_data_available_event);
+
+	pthread_mutex_unlock(&ffm->io.data_mutex);
+
+	return buf_size;
+}
+
+#define SRT_PROTO "srt"
+#define UDP_PROTO "udp"
+#define TCP_PROTO "tcp"
+#define HTTP_PROTO "http"
+#define RIST_PROTO "rist"
+
+static bool ffmpeg_mux_is_network(struct ffmpeg_mux *ffm)
+{
+	return !strncmp(ffm->params.file, SRT_PROTO, sizeof(SRT_PROTO) - 1) ||
+	       !strncmp(ffm->params.file, UDP_PROTO, sizeof(UDP_PROTO) - 1) ||
+	       !strncmp(ffm->params.file, TCP_PROTO, sizeof(TCP_PROTO) - 1) ||
+	       !strncmp(ffm->params.file, HTTP_PROTO, sizeof(HTTP_PROTO) - 1) ||
+	       !strncmp(ffm->params.file, RIST_PROTO, sizeof(RIST_PROTO) - 1);
+}
+
 static inline int open_output_file(struct ffmpeg_mux *ffm)
 {
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(59, 0, 100)
 	AVOutputFormat *format = ffm->output->oformat;
+#else
+	const AVOutputFormat *format = ffm->output->oformat;
+#endif
 	int ret;
 
 	if ((format->flags & AVFMT_NOFILE) == 0) {
-		ret = avio_open(&ffm->output->pb, ffm->params.file,
-				AVIO_FLAG_WRITE);
-		if (ret < 0) {
-			fprintf(stderr, "Couldn't open '%s', %s\n",
-				ffm->params.printable_file.array,
-				av_err2str(ret));
-			return FFM_ERROR;
+		if (!ffmpeg_mux_is_network(ffm)) {
+			// If not outputting to a network, write to a circlebuf
+			// instead of relying on ffmpeg disk output. This hopefully
+			// works around too small buffers somewhere causing output
+			// stalls when recording.
+
+			// We're in charge of managing the actual file now
+			ffm->io.output_file = os_fopen(ffm->params.file, "wb");
+			if (!ffm->io.output_file) {
+				fprintf(stderr, "Couldn't open '%s', %s\n",
+					ffm->params.printable_file.array,
+					strerror(errno));
+				return FFM_ERROR;
+			}
+
+			// Start at 1MB, this can grow up to 256 MB depending
+			// how fast data is going in and out (limited in
+			// ffmpeg_mux_write_av_buffer)
+			circlebuf_reserve(&ffm->io.data, 1048576);
+
+			pthread_mutex_init(&ffm->io.data_mutex, NULL);
+
+			os_event_init(&ffm->io.buffer_space_available_event,
+				      OS_EVENT_TYPE_AUTO);
+			os_event_init(&ffm->io.new_data_available_event,
+				      OS_EVENT_TYPE_AUTO);
+
+			pthread_create(&ffm->io.io_thread, NULL,
+				       ffmpeg_mux_io_thread, ffm);
+
+			unsigned char *avio_ctx_buffer =
+				av_malloc(AVIO_BUFFER_SIZE);
+
+			ffm->output->pb = avio_alloc_context(
+				avio_ctx_buffer, AVIO_BUFFER_SIZE, 1, ffm, NULL,
+				ffmpeg_mux_write_av_buffer,
+				ffmpeg_mux_seek_av_buffer);
+
+			ffm->io.active = true;
+		} else {
+			ret = avio_open(&ffm->output->pb, ffm->params.file,
+					AVIO_FLAG_WRITE);
+			if (ret < 0) {
+				fprintf(stderr, "Couldn't open '%s', %s\n",
+					ffm->params.printable_file.array,
+					av_err2str(ret));
+				return FFM_ERROR;
+			}
 		}
 	}
 
@@ -624,22 +991,13 @@ static inline int open_output_file(struct ffmpeg_mux *ffm)
 	return FFM_SUCCESS;
 }
 
-#define SRT_PROTO "srt"
-#define UDP_PROTO "udp"
-#define TCP_PROTO "tcp"
-#define HTTP_PROTO "http"
-
-static bool ffmpeg_mux_is_network(struct ffmpeg_mux *ffm)
-{
-	return !strncmp(ffm->params.file, SRT_PROTO, sizeof(SRT_PROTO) - 1) ||
-	       !strncmp(ffm->params.file, UDP_PROTO, sizeof(UDP_PROTO) - 1) ||
-	       !strncmp(ffm->params.file, TCP_PROTO, sizeof(TCP_PROTO) - 1) ||
-	       !strncmp(ffm->params.file, HTTP_PROTO, sizeof(HTTP_PROTO) - 1);
-}
-
 static int ffmpeg_mux_init_context(struct ffmpeg_mux *ffm)
 {
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(59, 0, 100)
 	AVOutputFormat *output_format;
+#else
+	const AVOutputFormat *output_format;
+#endif
 	int ret;
 	bool is_http = false;
 	is_http = (strncmp(ffm->params.file, HTTP_PROTO,
@@ -661,9 +1019,12 @@ static int ffmpeg_mux_init_context(struct ffmpeg_mux *ffm)
 			ffm->params.printable_file.array);
 		return FFM_ERROR;
 	}
+
+#ifdef ENABLE_FFMPEG_MUX_DEBUG
 	printf("info: Output format name and long_name: %s, %s\n",
 	       output_format->name ? output_format->name : "unknown",
 	       output_format->long_name ? output_format->long_name : "unknown");
+#endif
 
 	ret = avformat_alloc_output_context2(&ffm->output, output_format, NULL,
 					     ffm->params.file);
@@ -673,8 +1034,10 @@ static int ffmpeg_mux_init_context(struct ffmpeg_mux *ffm)
 		return FFM_ERROR;
 	}
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(59, 0, 100)
 	ffm->output->oformat->video_codec = AV_CODEC_ID_NONE;
 	ffm->output->oformat->audio_codec = AV_CODEC_ID_NONE;
+#endif
 
 	if (!init_streams(ffm)) {
 		free_avformat(ffm);
@@ -710,6 +1073,8 @@ static int ffmpeg_mux_init_internal(struct ffmpeg_mux *ffm, int argc,
 	if (!ffmpeg_mux_get_extra_data(ffm))
 		return FFM_ERROR;
 
+	ffm->packet = av_packet_alloc();
+
 	/* ffmpeg does not have a way of telling what's supported
 	 * for a given output format, so we try each possibility */
 	return ffmpeg_mux_init_context(ffm);
@@ -779,7 +1144,6 @@ static inline bool ffmpeg_mux_packet(struct ffmpeg_mux *ffm, uint8_t *buf,
 				     struct ffm_packet_info *info)
 {
 	int idx = get_index(ffm, info);
-	AVPacket packet = {0};
 
 	/* The muxer might not support video/audio, or multiple audio tracks */
 	if (idx == -1) {
@@ -789,32 +1153,61 @@ static inline bool ffmpeg_mux_packet(struct ffmpeg_mux *ffm, uint8_t *buf,
 	const AVRational codec_time_base =
 		get_codec_context(ffm, info)->time_base;
 
-	av_init_packet(&packet);
-
-	packet.data = buf;
-	packet.size = (int)info->size;
-	packet.stream_index = idx;
-	packet.pts = rescale_ts(ffm, codec_time_base, info->pts, idx);
-	packet.dts = rescale_ts(ffm, codec_time_base, info->dts, idx);
+	ffm->packet->data = buf;
+	ffm->packet->size = (int)info->size;
+	ffm->packet->stream_index = idx;
+	ffm->packet->pts = rescale_ts(ffm, codec_time_base, info->pts, idx);
+	ffm->packet->dts = rescale_ts(ffm, codec_time_base, info->dts, idx);
 
 	if (info->keyframe)
-		packet.flags = AV_PKT_FLAG_KEY;
+		ffm->packet->flags = AV_PKT_FLAG_KEY;
 
-	int ret = av_interleaved_write_frame(ffm->output, &packet);
-
-	if (ret < 0) {
-		fprintf(stderr, "av_interleaved_write_frame failed: %d: %s\n",
-			ret, av_err2str(ret));
-	}
+	int ret = av_interleaved_write_frame(ffm->output, ffm->packet);
 
 	/* Treat "Invalid data found when processing input" and "Invalid argument" as non-fatal */
 	if (ret == AVERROR_INVALIDDATA || ret == -EINVAL) {
 		return true;
 	}
 
+	if (ret < 0) {
+		fprintf(stderr, "av_interleaved_write_frame failed: %d: %s\n",
+			ret, av_err2str(ret));
+	}
+
 	return ret >= 0;
 }
 
+static inline bool read_change_file(struct ffmpeg_mux *ffm, uint32_t size,
+				    struct resize_buf *filename, int argc,
+				    char **argv)
+{
+	resize_buf_resize(filename, size + 1);
+	if (safe_read(filename->buf, size) != size) {
+		return false;
+	}
+	filename->buf[size] = 0;
+
+#ifdef ENABLE_FFMPEG_MUX_DEBUG
+	fprintf(stderr, "info: New output file name: %s\n", filename->buf);
+#endif
+
+	int ret;
+	char *argv1_backup = argv[1];
+	argv[1] = (char *)filename->buf;
+
+	ffmpeg_mux_free(ffm);
+
+	ret = ffmpeg_mux_init(ffm, argc, argv);
+	if (ret != FFM_SUCCESS) {
+		fprintf(stderr, "Couldn't initialize muxer\n");
+		return false;
+	}
+
+	argv[1] = argv1_backup;
+
+	return true;
+}
+
 /* ------------------------------------------------------------------------- */
 
 #ifdef _WIN32
@@ -826,6 +1219,7 @@ int main(int argc, char *argv[])
 	struct ffm_packet_info info = {0};
 	struct ffmpeg_mux ffm = {0};
 	struct resize_buf rb = {0};
+	struct resize_buf rb_filename = {0};
 	bool fail = false;
 	int ret;
 
@@ -858,6 +1252,12 @@ int main(int argc, char *argv[])
 	}
 
 	while (!fail && safe_read(&info, sizeof(info)) == sizeof(info)) {
+		if (info.type == FFM_PACKET_CHANGE_FILE) {
+			fail = !read_change_file(&ffm, info.size, &rb_filename,
+						 argc, argv);
+			continue;
+		}
+
 		resize_buf_resize(&rb, info.size);
 
 		if (safe_read(rb.buf, info.size) == info.size) {
@@ -869,6 +1269,7 @@ int main(int argc, char *argv[])
 
 	ffmpeg_mux_free(&ffm);
 	resize_buf_free(&rb);
+	resize_buf_free(&rb_filename);
 
 #ifdef _WIN32
 	for (int i = 0; i < argc; i++)
