@@ -8,6 +8,12 @@
 #include <dxgi.h>
 #include <d3d11.h>
 #include <d3d11_1.h>
+#ifdef ENABLE_HEVC
+#include <obs-hevc.h>
+#endif
+
+/* TODO: Use new preset scheme */
+#pragma warning(disable : 4996)
 
 /* ========================================================================= */
 
@@ -38,6 +44,9 @@ struct handle_tex {
 
 struct nvenc_data {
 	obs_encoder_t *encoder;
+#ifdef ENABLE_HEVC
+	bool hevc;
+#endif
 
 	void *session;
 	NV_ENC_INITIALIZE_PARAMS params;
@@ -80,57 +89,28 @@ struct nvenc_data {
 
 struct nv_bitstream {
 	void *ptr;
-	HANDLE event;
 };
 
+#define NV_FAIL(format, ...) nv_fail(enc->encoder, format, __VA_ARGS__)
 #define NV_FAILED(x) nv_failed(enc->encoder, x, __FUNCTION__, #x)
 
 static bool nv_bitstream_init(struct nvenc_data *enc, struct nv_bitstream *bs)
 {
 	NV_ENC_CREATE_BITSTREAM_BUFFER buf = {
 		NV_ENC_CREATE_BITSTREAM_BUFFER_VER};
-	NV_ENC_EVENT_PARAMS params = {NV_ENC_EVENT_PARAMS_VER};
-	HANDLE event = NULL;
 
 	if (NV_FAILED(nv.nvEncCreateBitstreamBuffer(enc->session, &buf))) {
 		return false;
 	}
 
-	event = CreateEvent(NULL, true, true, NULL);
-	if (!event) {
-		error("%s: %s", __FUNCTION__, "Failed to create event");
-		goto fail;
-	}
-
-	params.completionEvent = event;
-	if (NV_FAILED(nv.nvEncRegisterAsyncEvent(enc->session, &params))) {
-		goto fail;
-	}
-
 	bs->ptr = buf.bitstreamBuffer;
-	bs->event = event;
 	return true;
-
-fail:
-	if (event) {
-		CloseHandle(event);
-	}
-	if (buf.bitstreamBuffer) {
-		nv.nvEncDestroyBitstreamBuffer(enc->session,
-					       buf.bitstreamBuffer);
-	}
-	return false;
 }
 
 static void nv_bitstream_free(struct nvenc_data *enc, struct nv_bitstream *bs)
 {
 	if (bs->ptr) {
 		nv.nvEncDestroyBitstreamBuffer(enc->session, bs->ptr);
-
-		NV_ENC_EVENT_PARAMS params = {NV_ENC_EVENT_PARAMS_VER};
-		params.completionEvent = bs->event;
-		nv.nvEncUnregisterAsyncEvent(enc->session, &params);
-		CloseHandle(bs->event);
 	}
 }
 
@@ -145,20 +125,20 @@ struct nv_texture {
 
 static bool nv_texture_init(struct nvenc_data *enc, struct nv_texture *nvtex)
 {
-	ID3D11Device *device = enc->device;
-	ID3D11Texture2D *tex;
-	HRESULT hr;
+	const bool p010 = obs_p010_tex_active();
 
 	D3D11_TEXTURE2D_DESC desc = {0};
 	desc.Width = enc->cx;
 	desc.Height = enc->cy;
 	desc.MipLevels = 1;
 	desc.ArraySize = 1;
-	desc.Format = DXGI_FORMAT_NV12;
+	desc.Format = p010 ? DXGI_FORMAT_P010 : DXGI_FORMAT_NV12;
 	desc.SampleDesc.Count = 1;
 	desc.BindFlags = D3D11_BIND_RENDER_TARGET;
 
-	hr = device->lpVtbl->CreateTexture2D(device, &desc, NULL, &tex);
+	ID3D11Device *const device = enc->device;
+	ID3D11Texture2D *tex;
+	HRESULT hr = device->lpVtbl->CreateTexture2D(device, &desc, NULL, &tex);
 	if (FAILED(hr)) {
 		error_hr("Failed to create texture");
 		return false;
@@ -171,7 +151,8 @@ static bool nv_texture_init(struct nvenc_data *enc, struct nv_texture *nvtex)
 	res.resourceToRegister = tex;
 	res.width = enc->cx;
 	res.height = enc->cy;
-	res.bufferFormat = NV_ENC_BUFFER_FORMAT_NV12;
+	res.bufferFormat = p010 ? NV_ENC_BUFFER_FORMAT_YUV420_10BIT
+				: NV_ENC_BUFFER_FORMAT_NV12;
 
 	if (NV_FAILED(nv.nvEncRegisterResource(enc->session, &res))) {
 		tex->lpVtbl->Release(tex);
@@ -180,6 +161,7 @@ static bool nv_texture_init(struct nvenc_data *enc, struct nv_texture *nvtex)
 
 	nvtex->res = res.registeredResource;
 	nvtex->tex = tex;
+	nvtex->mapped_res = NULL;
 	return true;
 }
 
@@ -198,13 +180,22 @@ static void nv_texture_free(struct nvenc_data *enc, struct nv_texture *nvtex)
 /* ------------------------------------------------------------------------- */
 /* Implementation                                                            */
 
-static const char *nvenc_get_name(void *type_data)
+static const char *h264_nvenc_get_name(void *type_data)
 {
 	UNUSED_PARAMETER(type_data);
-	return "NVIDIA NVENC H.264 (new)";
+	return "NVIDIA NVENC H.264";
 }
 
-static inline int nv_get_cap(struct nvenc_data *enc, NV_ENC_CAPS cap)
+#ifdef ENABLE_HEVC
+static const char *hevc_nvenc_get_name(void *type_data)
+{
+	UNUSED_PARAMETER(type_data);
+	return "NVIDIA NVENC HEVC";
+}
+#endif
+
+static inline int nv_get_cap_internal(struct nvenc_data *enc, NV_ENC_CAPS cap,
+				      GUID encodeGUID)
 {
 	if (!enc->session)
 		return 0;
@@ -213,10 +204,22 @@ static inline int nv_get_cap(struct nvenc_data *enc, NV_ENC_CAPS cap)
 	int v;
 
 	param.capsToQuery = cap;
-	nv.nvEncGetEncodeCaps(enc->session, NV_ENC_CODEC_H264_GUID, &param, &v);
+	nv.nvEncGetEncodeCaps(enc->session, encodeGUID, &param, &v);
 	return v;
 }
 
+static inline int nv_get_cap_h264(struct nvenc_data *enc, NV_ENC_CAPS cap)
+{
+	return nv_get_cap_internal(enc, cap, NV_ENC_CODEC_H264_GUID);
+}
+
+#ifdef ENABLE_HEVC
+static inline int nv_get_cap_hevc(struct nvenc_data *enc, NV_ENC_CAPS cap)
+{
+	return nv_get_cap_internal(enc, cap, NV_ENC_CODEC_HEVC_GUID);
+}
+#endif
+
 static bool nvenc_update(void *data, obs_data_t *settings)
 {
 	struct nvenc_data *enc = data;
@@ -323,8 +326,31 @@ static bool init_session(struct nvenc_data *enc)
 	return true;
 }
 
-static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
-			 bool psycho_aq)
+static void initialize_params(NV_ENC_INITIALIZE_PARAMS *params,
+			      const GUID *nv_encode, const GUID *nv_preset,
+			      uint32_t width, uint32_t height, uint32_t fps_num,
+			      uint32_t fps_den, NV_ENC_CONFIG *config)
+{
+	int darWidth, darHeight;
+	av_reduce(&darWidth, &darHeight, width, height, 1024 * 1024);
+
+	memset(params, 0, sizeof(*params));
+	params->version = NV_ENC_INITIALIZE_PARAMS_VER;
+	params->encodeGUID = *nv_encode;
+	params->presetGUID = *nv_preset;
+	params->encodeWidth = width;
+	params->encodeHeight = height;
+	params->darWidth = darWidth;
+	params->darHeight = darHeight;
+	params->frameRateNum = fps_num;
+	params->frameRateDen = fps_den;
+	params->enableEncodeAsync = 0;
+	params->enablePTD = 1;
+	params->encodeConfig = config;
+}
+
+static bool init_encoder_h264(struct nvenc_data *enc, obs_data_t *settings,
+			      int bf, bool psycho_aq)
 {
 	const char *rc = obs_data_get_string(settings, "rate_control");
 	int bitrate = (int)obs_data_get_int(settings, "bitrate");
@@ -334,7 +360,6 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	const char *preset = obs_data_get_string(settings, "preset");
 	const char *profile = obs_data_get_string(settings, "profile");
 	bool lookahead = obs_data_get_bool(settings, "lookahead");
-	int bf = (int)obs_data_get_int(settings, "bf");
 	bool vbr = astrcmpi(rc, "VBR") == 0;
 	NVENCSTATUS err;
 
@@ -380,7 +405,8 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	const bool rc_lossless = astrcmpi(rc, "lossless") == 0;
 	bool lossless = rc_lossless;
 	if (rc_lossless) {
-		lossless = nv_get_cap(enc, NV_ENC_CAPS_SUPPORT_LOSSLESS_ENCODE);
+		lossless = nv_get_cap_h264(enc,
+					   NV_ENC_CAPS_SUPPORT_LOSSLESS_ENCODE);
 		if (lossless) {
 			nv_preset = hp ? NV_ENC_PRESET_LOSSLESS_HP_GUID
 				       : NV_ENC_PRESET_LOSSLESS_DEFAULT_GUID;
@@ -411,28 +437,14 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	uint32_t gop_size =
 		(keyint_sec) ? keyint_sec * voi->fps_num / voi->fps_den : 250;
 
-	NV_ENC_INITIALIZE_PARAMS *params = &enc->params;
 	NV_ENC_CONFIG *config = &enc->config;
 	NV_ENC_CONFIG_H264 *h264_config = &config->encodeCodecConfig.h264Config;
 	NV_ENC_CONFIG_H264_VUI_PARAMETERS *vui_params =
 		&h264_config->h264VUIParameters;
 
-	int darWidth, darHeight;
-	av_reduce(&darWidth, &darHeight, voi->width, voi->height, 1024 * 1024);
-
-	memset(params, 0, sizeof(*params));
-	params->version = NV_ENC_INITIALIZE_PARAMS_VER;
-	params->encodeGUID = NV_ENC_CODEC_H264_GUID;
-	params->presetGUID = nv_preset;
-	params->encodeWidth = voi->width;
-	params->encodeHeight = voi->height;
-	params->darWidth = darWidth;
-	params->darHeight = darHeight;
-	params->frameRateNum = voi->fps_num;
-	params->frameRateDen = voi->fps_den;
-	params->enableEncodeAsync = 1;
-	params->enablePTD = 1;
-	params->encodeConfig = &enc->config;
+	initialize_params(&enc->params, &NV_ENC_CODEC_H264_GUID, &nv_preset,
+			  voi->width, voi->height, voi->fps_num, voi->fps_den,
+			  &enc->config);
 	config->gopLength = gop_size;
 	config->frameIntervalP = 1 + bf;
 	h264_config->idrPeriod = gop_size;
@@ -476,7 +488,7 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 
 	/* lookahead */
 	const bool use_profile_lookahead = config->rcParams.enableLookahead;
-	lookahead = nv_get_cap(enc, NV_ENC_CAPS_SUPPORT_LOOKAHEAD) &&
+	lookahead = nv_get_cap_h264(enc, NV_ENC_CAPS_SUPPORT_LOOKAHEAD) &&
 		    (lookahead || use_profile_lookahead);
 	if (lookahead) {
 		enc->rc_lookahead = use_profile_lookahead
@@ -511,7 +523,7 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	}
 
 	/* psycho aq */
-	if (nv_get_cap(enc, NV_ENC_CAPS_SUPPORT_TEMPORAL_AQ)) {
+	if (nv_get_cap_h264(enc, NV_ENC_CAPS_SUPPORT_TEMPORAL_AQ)) {
 		config->rcParams.enableAQ = psycho_aq;
 		config->rcParams.aqStrength = 8;
 		config->rcParams.enableTemporalAQ = psycho_aq;
@@ -523,7 +535,7 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	/* rate control               */
 
 	enc->can_change_bitrate =
-		nv_get_cap(enc, NV_ENC_CAPS_SUPPORT_DYN_BITRATE_CHANGE) &&
+		nv_get_cap_h264(enc, NV_ENC_CAPS_SUPPORT_DYN_BITRATE_CHANGE) &&
 		!lookahead;
 
 	config->rcParams.rateControlMode = twopass ? NV_ENC_PARAMS_RC_VBR_HQ
@@ -570,11 +582,12 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	/* -------------------------- */
 	/* initialize                 */
 
-	if (NV_FAILED(nv.nvEncInitializeEncoder(enc->session, params))) {
+	if (NV_FAILED(nv.nvEncInitializeEncoder(enc->session, &enc->params))) {
 		return false;
 	}
 
 	info("settings:\n"
+	     "\tcodec:        H264\n"
 	     "\trate_control: %s\n"
 	     "\tbitrate:      %d\n"
 	     "\tcqp:          %d\n"
@@ -594,6 +607,282 @@ static bool init_encoder(struct nvenc_data *enc, obs_data_t *settings,
 	return true;
 }
 
+#ifdef ENABLE_HEVC
+static bool init_encoder_hevc(struct nvenc_data *enc, obs_data_t *settings,
+			      int bf, bool psycho_aq)
+{
+	const char *rc = obs_data_get_string(settings, "rate_control");
+	int bitrate = (int)obs_data_get_int(settings, "bitrate");
+	int max_bitrate = (int)obs_data_get_int(settings, "max_bitrate");
+	int cqp = (int)obs_data_get_int(settings, "cqp");
+	int keyint_sec = (int)obs_data_get_int(settings, "keyint_sec");
+	const char *preset = obs_data_get_string(settings, "preset");
+	const char *profile = obs_data_get_string(settings, "profile");
+	bool lookahead = obs_data_get_bool(settings, "lookahead");
+	bool vbr = astrcmpi(rc, "VBR") == 0;
+	NVENCSTATUS err;
+
+	video_t *video = obs_encoder_video(enc->encoder);
+	const struct video_output_info *voi = video_output_get_info(video);
+
+	enc->cx = voi->width;
+	enc->cy = voi->height;
+
+	/* -------------------------- */
+	/* get preset                 */
+
+	GUID nv_preset = NV_ENC_PRESET_DEFAULT_GUID;
+	bool twopass = false;
+	bool hp = false;
+	bool ll = false;
+
+	if (astrcmpi(preset, "hq") == 0) {
+		nv_preset = NV_ENC_PRESET_HQ_GUID;
+
+	} else if (astrcmpi(preset, "mq") == 0) {
+		nv_preset = NV_ENC_PRESET_HQ_GUID;
+		twopass = true;
+
+	} else if (astrcmpi(preset, "hp") == 0) {
+		nv_preset = NV_ENC_PRESET_HP_GUID;
+		hp = true;
+
+	} else if (astrcmpi(preset, "ll") == 0) {
+		nv_preset = NV_ENC_PRESET_LOW_LATENCY_DEFAULT_GUID;
+		ll = true;
+
+	} else if (astrcmpi(preset, "llhq") == 0) {
+		nv_preset = NV_ENC_PRESET_LOW_LATENCY_HQ_GUID;
+		ll = true;
+
+	} else if (astrcmpi(preset, "llhp") == 0) {
+		nv_preset = NV_ENC_PRESET_LOW_LATENCY_HP_GUID;
+		hp = true;
+		ll = true;
+	}
+
+	const bool rc_lossless = astrcmpi(rc, "lossless") == 0;
+	bool lossless = rc_lossless;
+	if (rc_lossless) {
+		lossless = nv_get_cap_hevc(enc,
+					   NV_ENC_CAPS_SUPPORT_LOSSLESS_ENCODE);
+		if (lossless) {
+			nv_preset = hp ? NV_ENC_PRESET_LOSSLESS_HP_GUID
+				       : NV_ENC_PRESET_LOSSLESS_DEFAULT_GUID;
+		} else {
+			warn("lossless encode is not supported, ignoring");
+		}
+	}
+
+	/* -------------------------- */
+	/* get preset default config  */
+
+	NV_ENC_PRESET_CONFIG preset_config = {NV_ENC_PRESET_CONFIG_VER,
+					      {NV_ENC_CONFIG_VER}};
+
+	err = nv.nvEncGetEncodePresetConfig(enc->session,
+					    NV_ENC_CODEC_HEVC_GUID, nv_preset,
+					    &preset_config);
+	if (nv_failed(enc->encoder, err, __FUNCTION__,
+		      "nvEncGetEncodePresetConfig")) {
+		return false;
+	}
+
+	/* -------------------------- */
+	/* main configuration         */
+
+	enc->config = preset_config.presetCfg;
+
+	uint32_t gop_size =
+		(keyint_sec) ? keyint_sec * voi->fps_num / voi->fps_den : 250;
+
+	NV_ENC_CONFIG *config = &enc->config;
+	NV_ENC_CONFIG_HEVC *hevc_config = &config->encodeCodecConfig.hevcConfig;
+	NV_ENC_CONFIG_HEVC_VUI_PARAMETERS *vui_params =
+		&hevc_config->hevcVUIParameters;
+
+	initialize_params(&enc->params, &NV_ENC_CODEC_HEVC_GUID, &nv_preset,
+			  voi->width, voi->height, voi->fps_num, voi->fps_den,
+			  &enc->config);
+	config->gopLength = gop_size;
+	config->frameIntervalP = 1 + bf;
+	hevc_config->idrPeriod = gop_size;
+
+	bool repeat_headers = obs_data_get_bool(settings, "repeat_headers");
+	if (repeat_headers) {
+		hevc_config->repeatSPSPPS = 1;
+		hevc_config->disableSPSPPS = 0;
+		hevc_config->outputAUD = 1;
+	}
+
+	hevc_config->sliceMode = 3;
+	hevc_config->sliceModeData = 1;
+
+	hevc_config->useBFramesAsRef = NV_ENC_BFRAME_REF_MODE_DISABLED;
+
+	vui_params->videoSignalTypePresentFlag = 1;
+	vui_params->videoFullRangeFlag = (voi->range == VIDEO_RANGE_FULL);
+	vui_params->colourDescriptionPresentFlag = 1;
+
+	hevc_config->pixelBitDepthMinus8 = obs_p010_tex_active() ? 2 : 0;
+
+	switch (voi->colorspace) {
+	case VIDEO_CS_601:
+		vui_params->colourPrimaries = 6;
+		vui_params->transferCharacteristics = 6;
+		vui_params->colourMatrix = 6;
+		break;
+	case VIDEO_CS_DEFAULT:
+	case VIDEO_CS_709:
+		vui_params->colourPrimaries = 1;
+		vui_params->transferCharacteristics = 1;
+		vui_params->colourMatrix = 1;
+		break;
+	case VIDEO_CS_SRGB:
+		vui_params->colourPrimaries = 1;
+		vui_params->transferCharacteristics = 13;
+		vui_params->colourMatrix = 1;
+		break;
+	case VIDEO_CS_2100_PQ:
+		vui_params->colourPrimaries = 9;
+		vui_params->transferCharacteristics = 16;
+		vui_params->colourMatrix = 9;
+		vui_params->chromaSampleLocationFlag = 1;
+		vui_params->chromaSampleLocationTop = 2;
+		vui_params->chromaSampleLocationBot = 2;
+		break;
+	case VIDEO_CS_2100_HLG:
+		vui_params->colourPrimaries = 9;
+		vui_params->transferCharacteristics = 18;
+		vui_params->colourMatrix = 9;
+		vui_params->chromaSampleLocationFlag = 1;
+		vui_params->chromaSampleLocationTop = 2;
+		vui_params->chromaSampleLocationBot = 2;
+	}
+
+	enc->bframes = bf;
+
+	/* lookahead */
+	const bool use_profile_lookahead = config->rcParams.enableLookahead;
+	lookahead = nv_get_cap_hevc(enc, NV_ENC_CAPS_SUPPORT_LOOKAHEAD) &&
+		    (lookahead || use_profile_lookahead);
+	if (lookahead) {
+		enc->rc_lookahead = use_profile_lookahead
+					    ? config->rcParams.lookaheadDepth
+					    : 8;
+	}
+
+	int buf_count = max(4, config->frameIntervalP * 2 * 2);
+	if (lookahead) {
+		buf_count = max(buf_count, config->frameIntervalP +
+						   enc->rc_lookahead +
+						   EXTRA_BUFFERS);
+	}
+
+	buf_count = min(64, buf_count);
+	enc->buf_count = buf_count;
+
+	const int output_delay = buf_count - 1;
+	enc->output_delay = output_delay;
+
+	if (lookahead) {
+		const int lkd_bound = output_delay - config->frameIntervalP - 4;
+		if (lkd_bound >= 0) {
+			config->rcParams.enableLookahead = 1;
+			config->rcParams.lookaheadDepth =
+				max(enc->rc_lookahead, lkd_bound);
+			config->rcParams.disableIadapt = 0;
+			config->rcParams.disableBadapt = 0;
+		} else {
+			lookahead = false;
+		}
+	}
+
+	/* psycho aq */
+	if (nv_get_cap_hevc(enc, NV_ENC_CAPS_SUPPORT_TEMPORAL_AQ)) {
+		config->rcParams.enableAQ = psycho_aq;
+		config->rcParams.aqStrength = 8;
+		config->rcParams.enableTemporalAQ = psycho_aq;
+	} else if (psycho_aq) {
+		warn("Ignoring Psycho Visual Tuning request since GPU is not capable");
+	}
+
+	/* -------------------------- */
+	/* rate control               */
+
+	enc->can_change_bitrate =
+		nv_get_cap_hevc(enc, NV_ENC_CAPS_SUPPORT_DYN_BITRATE_CHANGE) &&
+		!lookahead;
+
+	config->rcParams.rateControlMode = twopass ? NV_ENC_PARAMS_RC_VBR_HQ
+						   : NV_ENC_PARAMS_RC_VBR;
+
+	if (astrcmpi(rc, "cqp") == 0 || rc_lossless) {
+		if (lossless)
+			cqp = 0;
+
+		config->rcParams.rateControlMode = NV_ENC_PARAMS_RC_CONSTQP;
+		config->rcParams.constQP.qpInterP = cqp;
+		config->rcParams.constQP.qpInterB = cqp;
+		config->rcParams.constQP.qpIntra = cqp;
+		enc->can_change_bitrate = false;
+
+		bitrate = 0;
+		max_bitrate = 0;
+
+	} else if (astrcmpi(rc, "vbr") != 0) { /* CBR by default */
+		hevc_config->outputBufferingPeriodSEI = 1;
+		config->rcParams.rateControlMode =
+			twopass ? NV_ENC_PARAMS_RC_2_PASS_QUALITY
+				: NV_ENC_PARAMS_RC_CBR;
+	}
+
+	hevc_config->outputPictureTimingSEI = 1;
+	config->rcParams.averageBitRate = bitrate * 1000;
+	config->rcParams.maxBitRate = vbr ? max_bitrate * 1000 : bitrate * 1000;
+	config->rcParams.vbvBufferSize = bitrate * 1000;
+
+	/* -------------------------- */
+	/* profile                    */
+
+	if (astrcmpi(profile, "main10") == 0) {
+		config->profileGUID = NV_ENC_HEVC_PROFILE_MAIN10_GUID;
+	} else if (obs_p010_tex_active()) {
+		blog(LOG_WARNING, "[jim-nvenc] Forcing main10 for P010");
+		config->profileGUID = NV_ENC_HEVC_PROFILE_MAIN10_GUID;
+	} else {
+		config->profileGUID = NV_ENC_HEVC_PROFILE_MAIN_GUID;
+	}
+
+	/* -------------------------- */
+	/* initialize                 */
+
+	if (NV_FAILED(nv.nvEncInitializeEncoder(enc->session, &enc->params))) {
+		return false;
+	}
+
+	info("settings:\n"
+	     "\tcodec:        HEVC\n"
+	     "\trate_control: %s\n"
+	     "\tbitrate:      %d\n"
+	     "\tcqp:          %d\n"
+	     "\tkeyint:       %d\n"
+	     "\tpreset:       %s\n"
+	     "\tprofile:      %s\n"
+	     "\twidth:        %d\n"
+	     "\theight:       %d\n"
+	     "\t2-pass:       %s\n"
+	     "\tb-frames:     %d\n"
+	     "\tlookahead:    %s\n"
+	     "\tpsycho_aq:    %s\n",
+	     rc, bitrate, cqp, gop_size, preset, profile, enc->cx, enc->cy,
+	     twopass ? "true" : "false", bf, lookahead ? "true" : "false",
+	     psycho_aq ? "true" : "false");
+
+	return true;
+}
+#endif
+
 static bool init_bitstreams(struct nvenc_data *enc)
 {
 	da_reserve(enc->bitstreams, enc->buf_count);
@@ -611,7 +900,7 @@ static bool init_bitstreams(struct nvenc_data *enc)
 
 static bool init_textures(struct nvenc_data *enc)
 {
-	da_reserve(enc->bitstreams, enc->buf_count);
+	da_reserve(enc->textures, enc->buf_count);
 	for (int i = 0; i < enc->buf_count; i++) {
 		struct nv_texture texture;
 		if (!nv_texture_init(enc, &texture)) {
@@ -626,8 +915,79 @@ static bool init_textures(struct nvenc_data *enc)
 
 static void nvenc_destroy(void *data);
 
-static void *nvenc_create_internal(obs_data_t *settings, obs_encoder_t *encoder,
-				   bool psycho_aq)
+static bool init_specific_encoder(struct nvenc_data *enc, bool hevc,
+				  obs_data_t *settings, int bf, bool psycho_aq)
+{
+#ifdef ENABLE_HEVC
+	if (hevc)
+		return init_encoder_hevc(enc, settings, bf, psycho_aq);
+#endif
+
+	return init_encoder_h264(enc, settings, bf, psycho_aq);
+}
+
+static bool init_encoder(struct nvenc_data *enc, bool hevc,
+			 obs_data_t *settings)
+{
+	const int bf = (int)obs_data_get_int(settings, "bf");
+	const bool psycho_aq = obs_data_get_bool(settings, "psycho_aq");
+#ifdef ENABLE_HEVC
+	const bool support_10bit =
+		hevc ? nv_get_cap_hevc(enc, NV_ENC_CAPS_SUPPORT_10BIT_ENCODE)
+		     : nv_get_cap_h264(enc, NV_ENC_CAPS_SUPPORT_10BIT_ENCODE);
+	const int bf_max =
+		hevc ? nv_get_cap_hevc(enc, NV_ENC_CAPS_NUM_MAX_BFRAMES)
+		     : nv_get_cap_h264(enc, NV_ENC_CAPS_NUM_MAX_BFRAMES);
+#else
+	const bool support_10bit =
+		nv_get_cap_h264(enc, NV_ENC_CAPS_SUPPORT_10BIT_ENCODE);
+	const int bf_max = nv_get_cap_h264(enc, NV_ENC_CAPS_NUM_MAX_BFRAMES);
+#endif
+
+	if (obs_p010_tex_active() && !support_10bit) {
+		NV_FAIL(obs_module_text("NVENC.10bitUnsupported"));
+		return false;
+	}
+
+	video_t *video = obs_encoder_video(enc->encoder);
+	const struct video_output_info *voi = video_output_get_info(video);
+	switch (voi->format) {
+	case VIDEO_FORMAT_I010:
+	case VIDEO_FORMAT_P010:
+		break;
+	default:
+		switch (voi->colorspace) {
+		case VIDEO_CS_2100_PQ:
+		case VIDEO_CS_2100_HLG:
+			NV_FAIL(obs_module_text("NVENC.8bitUnsupportedHdr"));
+			return false;
+		}
+	}
+
+	if (bf > bf_max) {
+		NV_FAIL(obs_module_text("NVENC.TooManyBFrames"), bf, bf_max);
+		return false;
+	}
+
+	if (!init_specific_encoder(enc, hevc, settings, bf, psycho_aq)) {
+		if (!psycho_aq)
+			return false;
+
+		blog(LOG_WARNING, "[jim-nvenc] init_specific_encoder failed, "
+				  "trying again without Psycho Visual Tuning");
+		nv.nvEncDestroyEncoder(enc->session);
+		enc->session = NULL;
+		if (!init_session(enc) ||
+		    !init_specific_encoder(enc, hevc, settings, bf, false)) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
+static void *nvenc_create_internal(bool hevc, obs_data_t *settings,
+				   obs_encoder_t *encoder)
 {
 	NV_ENCODE_API_FUNCTION_LIST init = {NV_ENCODE_API_FUNCTION_LIST_VER};
 	struct nvenc_data *enc = bzalloc(sizeof(*enc));
@@ -646,7 +1006,7 @@ static void *nvenc_create_internal(obs_data_t *settings, obs_encoder_t *encoder,
 	if (!init_session(enc)) {
 		goto fail;
 	}
-	if (!init_encoder(enc, settings, psycho_aq)) {
+	if (!init_encoder(enc, hevc, settings)) {
 		goto fail;
 	}
 	if (!init_bitstreams(enc)) {
@@ -656,6 +1016,9 @@ static void *nvenc_create_internal(obs_data_t *settings, obs_encoder_t *encoder,
 		goto fail;
 	}
 
+#ifdef ENABLE_HEVC
+	enc->hevc = hevc;
+#endif
 	return enc;
 
 fail:
@@ -663,7 +1026,8 @@ fail:
 	return NULL;
 }
 
-static void *nvenc_create(obs_data_t *settings, obs_encoder_t *encoder)
+static void *nvenc_create_h264_hevc(bool hevc, obs_data_t *settings,
+				    obs_encoder_t *encoder)
 {
 	/* this encoder requires shared textures, this cannot be used on a
 	 * gpu other than the one OBS is currently running on. */
@@ -680,29 +1044,39 @@ static void *nvenc_create(obs_data_t *settings, obs_encoder_t *encoder)
 		goto reroute;
 	}
 
-	if (!obs_nv12_tex_active()) {
+	if (!obs_p010_tex_active() && !obs_nv12_tex_active()) {
 		blog(LOG_INFO,
-		     "[jim-nvenc] nv12 not active, falling back to ffmpeg");
+		     "[jim-nvenc] nv12/p010 not active, falling back to ffmpeg");
 		goto reroute;
 	}
 
-	const bool psycho_aq = obs_data_get_bool(settings, "psycho_aq");
-	struct nvenc_data *enc =
-		nvenc_create_internal(settings, encoder, psycho_aq);
-	if ((enc == NULL) && psycho_aq) {
-		blog(LOG_WARNING, "[jim-nvenc] nvenc_create_internal failed, "
-				  "trying again without Psycho Visual Tuning");
-		enc = nvenc_create_internal(settings, encoder, false);
-	}
+	struct nvenc_data *enc = nvenc_create_internal(hevc, settings, encoder);
 
 	if (enc) {
 		return enc;
 	}
 
 reroute:
-	return obs_encoder_create_rerouted(encoder, "ffmpeg_nvenc");
+	const char *fallback_name = "ffmpeg_nvenc";
+#ifdef ENABLE_HEVC
+	if (hevc)
+		fallback_name = "ffmpeg_hevc_nvenc";
+#endif
+	return obs_encoder_create_rerouted(encoder, fallback_name);
 }
 
+static void *h264_nvenc_create(obs_data_t *settings, obs_encoder_t *encoder)
+{
+	return nvenc_create_h264_hevc(false, settings, encoder);
+}
+
+#ifdef ENABLE_HEVC
+static void *hevc_nvenc_create(obs_data_t *settings, obs_encoder_t *encoder)
+{
+	return nvenc_create_h264_hevc(true, settings, encoder);
+}
+#endif
+
 static bool get_encoded_packet(struct nvenc_data *enc, bool finalize);
 
 static void nvenc_destroy(void *data)
@@ -711,11 +1085,9 @@ static void nvenc_destroy(void *data)
 
 	if (enc->encode_started) {
 		size_t next_bitstream = enc->next_bitstream;
-		HANDLE next_event = enc->bitstreams.array[next_bitstream].event;
 
 		NV_ENC_PIC_PARAMS params = {NV_ENC_PIC_PARAMS_VER};
 		params.encodePicFlags = NV_ENC_PIC_FLAG_EOS;
-		params.completionEvent = next_event;
 		nv.nvEncEncodePicture(enc->session, &params);
 		get_encoded_packet(enc, true);
 	}
@@ -828,11 +1200,22 @@ static bool get_encoded_packet(struct nvenc_data *enc, bool finalize)
 			size_t size;
 
 			enc->first_packet = false;
-			obs_extract_avc_headers(lock.bitstreamBufferPtr,
-						lock.bitstreamSizeInBytes,
-						&new_packet, &size,
-						&enc->header, &enc->header_size,
-						&enc->sei, &enc->sei_size);
+#ifdef ENABLE_HEVC
+			if (enc->hevc) {
+				obs_extract_hevc_headers(
+					lock.bitstreamBufferPtr,
+					lock.bitstreamSizeInBytes, &new_packet,
+					&size, &enc->header, &enc->header_size,
+					&enc->sei, &enc->sei_size);
+			} else
+#endif
+			{
+				obs_extract_avc_headers(
+					lock.bitstreamBufferPtr,
+					lock.bitstreamSizeInBytes, &new_packet,
+					&size, &enc->header, &enc->header_size,
+					&enc->sei, &enc->sei_size);
+			}
 
 			da_copy_array(enc->packet_data, new_packet, size);
 			bfree(new_packet);
@@ -905,11 +1288,6 @@ static bool nvenc_encode_tex(void *data, uint32_t handle, int64_t pts,
 
 	circlebuf_push_back(&enc->dts_list, &pts, sizeof(pts));
 
-	/* ------------------------------------ */
-	/* wait for output bitstream/tex        */
-
-	WaitForSingleObject(bs->event, INFINITE);
-
 	/* ------------------------------------ */
 	/* copy to output tex                   */
 
@@ -938,12 +1316,14 @@ static bool nvenc_encode_tex(void *data, uint32_t handle, int64_t pts,
 	params.version = NV_ENC_PIC_PARAMS_VER;
 	params.pictureStruct = NV_ENC_PIC_STRUCT_FRAME;
 	params.inputBuffer = nvtex->mapped_res;
-	params.bufferFmt = NV_ENC_BUFFER_FORMAT_NV12;
+	params.bufferFmt = obs_p010_tex_active()
+				   ? NV_ENC_BUFFER_FORMAT_YUV420_10BIT
+				   : NV_ENC_BUFFER_FORMAT_NV12;
 	params.inputTimeStamp = (uint64_t)pts;
 	params.inputWidth = enc->cx;
 	params.inputHeight = enc->cy;
+	params.inputPitch = enc->cx;
 	params.outputBitstream = bs->ptr;
-	params.completionEvent = bs->event;
 
 	err = nv.nvEncEncodePicture(enc->session, &params);
 	if (err != NV_ENC_SUCCESS && err != NV_ENC_ERR_NEED_MORE_INPUT) {
@@ -990,8 +1370,12 @@ static bool nvenc_encode_tex(void *data, uint32_t handle, int64_t pts,
 	return true;
 }
 
-extern void nvenc_defaults(obs_data_t *settings);
-extern obs_properties_t *nvenc_properties(void *unused);
+extern void h264_nvenc_defaults(obs_data_t *settings);
+extern obs_properties_t *h264_nvenc_properties(void *unused);
+#ifdef ENABLE_HEVC
+extern void hevc_nvenc_defaults(obs_data_t *settings);
+extern obs_properties_t *hevc_nvenc_properties(void *unused);
+#endif
 
 static bool nvenc_extra_data(void *data, uint8_t **header, size_t *size)
 {
@@ -1019,18 +1403,36 @@ static bool nvenc_sei_data(void *data, uint8_t **sei, size_t *size)
 	return true;
 }
 
-struct obs_encoder_info nvenc_info = {
+struct obs_encoder_info h264_nvenc_info = {
 	.id = "jim_nvenc",
 	.codec = "h264",
 	.type = OBS_ENCODER_VIDEO,
 	.caps = OBS_ENCODER_CAP_PASS_TEXTURE | OBS_ENCODER_CAP_DYN_BITRATE,
-	.get_name = nvenc_get_name,
-	.create = nvenc_create,
+	.get_name = h264_nvenc_get_name,
+	.create = h264_nvenc_create,
 	.destroy = nvenc_destroy,
 	.update = nvenc_update,
 	.encode_texture = nvenc_encode_tex,
-	.get_defaults = nvenc_defaults,
-	.get_properties = nvenc_properties,
+	.get_defaults = h264_nvenc_defaults,
+	.get_properties = h264_nvenc_properties,
 	.get_extra_data = nvenc_extra_data,
 	.get_sei_data = nvenc_sei_data,
 };
+
+#ifdef ENABLE_HEVC
+struct obs_encoder_info hevc_nvenc_info = {
+	.id = "jim_hevc_nvenc",
+	.codec = "hevc",
+	.type = OBS_ENCODER_VIDEO,
+	.caps = OBS_ENCODER_CAP_PASS_TEXTURE | OBS_ENCODER_CAP_DYN_BITRATE,
+	.get_name = hevc_nvenc_get_name,
+	.create = hevc_nvenc_create,
+	.destroy = nvenc_destroy,
+	.update = nvenc_update,
+	.encode_texture = nvenc_encode_tex,
+	.get_defaults = hevc_nvenc_defaults,
+	.get_properties = hevc_nvenc_properties,
+	.get_extra_data = nvenc_extra_data,
+	.get_sei_data = nvenc_sei_data,
+};
+#endif
