@@ -15,6 +15,8 @@ obs_to_ffmpeg_video_format(enum video_format format)
 	switch (format) {
 	case VIDEO_FORMAT_I444:
 		return AV_PIX_FMT_YUV444P;
+	case VIDEO_FORMAT_I412:
+		return AV_PIX_FMT_YUV444P12LE;
 	case VIDEO_FORMAT_I420:
 		return AV_PIX_FMT_YUV420P;
 	case VIDEO_FORMAT_NV12:
@@ -23,6 +25,8 @@ obs_to_ffmpeg_video_format(enum video_format format)
 		return AV_PIX_FMT_YUYV422;
 	case VIDEO_FORMAT_UYVY:
 		return AV_PIX_FMT_UYVY422;
+	case VIDEO_FORMAT_YVYU:
+		return AV_PIX_FMT_YVYU422;
 	case VIDEO_FORMAT_RGBA:
 		return AV_PIX_FMT_RGBA;
 	case VIDEO_FORMAT_BGRA:
@@ -35,14 +39,25 @@ obs_to_ffmpeg_video_format(enum video_format format)
 		return AV_PIX_FMT_BGR24;
 	case VIDEO_FORMAT_I422:
 		return AV_PIX_FMT_YUV422P;
+	case VIDEO_FORMAT_I210:
+		return AV_PIX_FMT_YUV422P10LE;
 	case VIDEO_FORMAT_I40A:
 		return AV_PIX_FMT_YUVA420P;
 	case VIDEO_FORMAT_I42A:
 		return AV_PIX_FMT_YUVA422P;
 	case VIDEO_FORMAT_YUVA:
 		return AV_PIX_FMT_YUVA444P;
+	case VIDEO_FORMAT_YA2L:
+#if LIBAVUTIL_BUILD >= AV_VERSION_INT(56, 31, 100)
+		return AV_PIX_FMT_YUVA444P12LE;
+#else
+		return AV_PIX_FMT_NONE;
+#endif
+	case VIDEO_FORMAT_I010:
+		return AV_PIX_FMT_YUV420P10LE;
+	case VIDEO_FORMAT_P010:
+		return AV_PIX_FMT_P010LE;
 	case VIDEO_FORMAT_NONE:
-	case VIDEO_FORMAT_YVYU:
 	case VIDEO_FORMAT_AYUV:
 		/* not supported by FFmpeg */
 		return AV_PIX_FMT_NONE;
@@ -51,42 +66,6 @@ obs_to_ffmpeg_video_format(enum video_format format)
 	return AV_PIX_FMT_NONE;
 }
 
-static inline enum video_format
-ffmpeg_to_obs_video_format(enum AVPixelFormat format)
-{
-	switch (format) {
-	case AV_PIX_FMT_YUV444P:
-		return VIDEO_FORMAT_I444;
-	case AV_PIX_FMT_YUV420P:
-		return VIDEO_FORMAT_I420;
-	case AV_PIX_FMT_NV12:
-		return VIDEO_FORMAT_NV12;
-	case AV_PIX_FMT_YUYV422:
-		return VIDEO_FORMAT_YUY2;
-	case AV_PIX_FMT_UYVY422:
-		return VIDEO_FORMAT_UYVY;
-	case AV_PIX_FMT_RGBA:
-		return VIDEO_FORMAT_RGBA;
-	case AV_PIX_FMT_BGRA:
-		return VIDEO_FORMAT_BGRA;
-	case AV_PIX_FMT_GRAY8:
-		return VIDEO_FORMAT_Y800;
-	case AV_PIX_FMT_BGR24:
-		return VIDEO_FORMAT_BGR3;
-	case AV_PIX_FMT_YUV422P:
-		return VIDEO_FORMAT_I422;
-	case AV_PIX_FMT_YUVA420P:
-		return VIDEO_FORMAT_I40A;
-	case AV_PIX_FMT_YUVA422P:
-		return VIDEO_FORMAT_I42A;
-	case AV_PIX_FMT_YUVA444P:
-		return VIDEO_FORMAT_YUVA;
-	case AV_PIX_FMT_NONE:
-	default:
-		return VIDEO_FORMAT_NONE;
-	}
-}
-
 static inline enum audio_format
 convert_ffmpeg_sample_format(enum AVSampleFormat format)
 {
